/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobKontoauszug;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * SynchronizeJobProvider-Abstract class for a Mahoe job
 *
 * @author Frank Mnster
 */
public abstract class AbstractMahoeSynchronizeJobProviderKontoauszug implements MahoeSynchronizeJobProvider {

    /**
     * The back end defined as a resource
     */
    @Resource
    private final MahoeSynchronizeBackend backend = null; // NOPMD

    /**
     * Die vom Job-Provider explizit unterstuetzten Konto-Arten. Alle anderen
     * machen andere Provider.
     */
    private final Set<KontoType> supportedAccountTypes = new HashSet<>(); // NOPMD

    /**
     * Liste der von diesem Backend implementierten Jobs
     */
    private final List<Class<? extends SynchronizeJob>> jobs = new ArrayList<>();

    /**
     * Default constructor
     */
    public AbstractMahoeSynchronizeJobProviderKontoauszug() {
        // add jobs and account types
        this.jobs.addAll(this.getJobs());
        this.supportedAccountTypes.addAll(this.getSupportedAccountTypes());
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final Object arg0) {

        // Umsaetze und Salden werden zum Schluss ausgefuehrt,
        // damit die oben gesendeten Ueberweisungen gleich mit
        // erscheinen, insofern die Bank das untersttzt.
        return 1;
    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getJobTypes()
     */
    @Override
    public List<Class<? extends SynchronizeJob>> getJobTypes() {

        return this.jobs;
    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getSynchronizeJobs(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public List<SynchronizeJob> getSynchronizeJobs(final Konto konto) {

        final Class<SynchronizeJobKontoauszug> type = SynchronizeJobKontoauszug.class;

        final List<SynchronizeJob> myJobs = new LinkedList<>();
        for (final Konto kto : this.backend.getSynchronizeKonten(konto)) {
            try {
                // Is Mahoe Backend and account supported?
                if (this.backend.supports(type, konto) && this.supports(type, konto)) {

                    final SynchronizeOptions options = new SynchronizeOptions(kto); // NOPMD
                    // Sync-Option zum Kontoauszugs-Abruf aktiv?
                    if (options.getSyncKontoauszuege()) {
                        // erzeugt eine Instanz von SynchronizeJobKontoauszug
                        final SynchronizeJobKontoauszug job = this.backend.create(type, kto);
                        job.setContext(SynchronizeJob.CTX_ENTITY, kto);
                        myJobs.add(job);
                    }
                }
            } catch (RemoteException | ApplicationException e) {
                Logger.error(I18N.tr("BshSynchronizeJobProviderKontoauszug.getSynchnizedJobsFailed"), e); //$NON-NLS-1$
            }
        }

        return myJobs;
    }

    /**
     * @see SynchronizeJobProvider#supports(Class, Konto)
     */
    @Override
    public boolean supports(final Class<? extends SynchronizeJob> type, final Konto konto) {

        boolean support = false;
        // Kein Konto angegeben. Dann gehen wir mal davon aus, dass es geht
        if (konto == null) {
            support = true;
        } else {
            KontoType kontoType = null;
            try {
                // Checken, ob es ein Aktien-Konto ist
                if (this.isBankSupported(konto)) {
                    // Checken, ob das vielleicht ein nicht unterstuetzter
                    // Konto-Typ ist
                    kontoType = KontoType.find(konto.getAccountType());
                    // Wenn kein konkreter Typ angegeben ist, dann unterstuetzen
                    // wir es
                    if (kontoType == null || this.supportedAccountTypes.contains(kontoType)) {
                        support = true;
                    }
                }
            } catch (final RemoteException re) {
                Logger.error(
                        I18N.tr("AbstractMahoeSynchronizeJobProviderKontoauszug.determineSupportForAccountTypeFailed") //$NON-NLS-1$
                                + kontoType,
                        re);
            }
        }
        return support;
    }

    /**
     * Returns the list of synchronise jobs to that can be executed.
     *
     * @return the list of synchronise jobs to that can be executed
     */
    protected abstract List<Class<? extends SynchronizeJob>> getJobs();

    /**
     * Returns the list of supported an account type.
     *
     * @return the list of supported an account type
     */
    protected abstract Set<KontoType> getSupportedAccountTypes();

    /**
     * Checks if the bank of a given account is supported in terms of BLZ or
     * BIC.
     *
     * @param account
     *            to check
     * @return <code>true</code> in case the BLZ or BIC is supported,
     *         <code>false</code> else.
     * @throws RemoteException
     */
    protected abstract boolean isBankSupported(final Konto account) throws RemoteException;

}
