/*******************************************************************************
 * Copyright 2018 fmCreation
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.aktien.gui.menu;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.aktien.gui.parts.AktienListEntry;
import de.frankmuenster.hibiscus.mahoe.aktien.gui.parts.AktienSettingsAktienTabellePart;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.CheckedContextMenuItem;
import de.willuhn.jameica.gui.parts.ContextMenu;
import de.willuhn.util.ApplicationException;

/**
 * The context menu for the stock table in the
 * {@link AktienSettingsAktienTabellePart}
 *
 * @author Frank Mnster
 */
public class AktienSettingsAktienTabelleContextMenu extends ContextMenu {

    /**
     * Die Listener
     */
    private final transient Set<AktienTabelleContextMenuListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Standardkonstruktor
     */
    public AktienSettingsAktienTabelleContextMenu() {

        super();
        // CheckedContextMenuItems will be disabled, if the user clicks into an
        // empty space of the table
        this.addItem(new CheckedContextMenuItem(I18N.tr("_change"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                if (context instanceof AktienListEntry) {
                    final AktienListEntry entry = (AktienListEntry) context;
                    if (StringUtils.isNotEmpty(entry.getStockId())) {
                        AktienSettingsAktienTabelleContextMenu.this.notifyListeners(entry);
                    }

                }

            }

        }));
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public final void addListener(final AktienTabelleContextMenuListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public final void removeListener(final AktienTabelleContextMenuListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen
     * Strings
     *
     * @param entry
     */
    private void notifyListeners(final AktienListEntry entry) {

        for (final AktienTabelleContextMenuListener listener : this.listeners) {
            listener.notifyStockToUpdateSelected(entry);
        }
    }

}
