/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.aktien.gui.parts;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

/**
 * TooltipListener is a Listener that creates a tool-tip for an {@link Item} in
 * a Table or a Tree. The code has been copied and modified from "Snippet 125"
 * of Eclipse, i.e.: copied and modified from
 * http://dev.eclipse.org/viewcvs/index.cgi/org.eclipse.swt.snippets/src/org/eclipse/swt/snippets/Snippet125.java
 *
 * @author keyser/Frank Mnster
 */
abstract class AbstractTooltipListener implements Listener {

    /**
     * The composite of the underlying table/tree
     */
    protected final transient Composite comp;

    /**
     * The display the composite is shown in
     */
    protected final transient Display display;

    /**
     * The actual label for the tool tip
     */
    protected transient Label label;

    /**
     * The listener handling mouse events
     */
    protected final transient Listener labelListener = new Listener() {

	/**
	 * @see Listener#handleEvent(Event)
	 */
	@Override
	public void handleEvent(final Event event) {

	    if (event.type == SWT.MouseDown) {
		final Item item = (Item) AbstractTooltipListener.this.label.getData("_TABLEITEM"); //$NON-NLS-1$
		AbstractTooltipListener.this.selectItem(item);
		final Event eventNew = new Event();
		eventNew.item = item;
		AbstractTooltipListener.this.comp.notifyListeners(SWT.Selection, eventNew);
		AbstractTooltipListener.this.disposeTip();
	    } else if (event.type == SWT.MouseExit) {
		AbstractTooltipListener.this.disposeTip();
	    }
	}
    };

    /**
     * The shell we are in
     */
    protected final transient Shell shell;

    /**
     * The shell for the tool tip
     */
    protected transient Shell tip;

    /**
     * @param comp
     *            Composite, i.e., a Table or a Tree
     */
    protected AbstractTooltipListener(final Composite comp) {
	this.shell = comp.getShell();
	this.display = this.shell.getDisplay();
	this.comp = comp;
    }

    /**
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt
     *      .widgets.Event)
     */
    @Override
    public void handleEvent(final Event event) {

	switch (event.type) {
	case SWT.Dispose:
	case SWT.KeyDown:
	case SWT.MouseMove:
	    this.disposeTip();
	    break;

	case SWT.MouseHover:
	    final Item item = this.retrieveItem(event);
	    if (item != null) {
		this.disposeTip(); // clear out old tip, if any
		this.createTipFor(item);
	    }
	    break;

	default:

	}
    }

    /**
     * Compute the text to be displayed for the given Item
     *
     * @param item
     *            Item
     * @return String
     */
    protected abstract String computeTextFor(final Item item);

    /**
     * Create the tooltip for the given Item
     *
     * @param item
     *            Item
     */
    protected void createTipFor(final Item item) {

	this.tip = new Shell(this.shell, SWT.ON_TOP | SWT.TOOL);
	this.tip.setLayout(new FillLayout());
	this.label = new Label(this.tip, SWT.NONE);
	this.label.setForeground(this.display.getSystemColor(SWT.COLOR_INFO_FOREGROUND));
	this.label.setBackground(this.display.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
	this.label.setText(this.computeTextFor(item));
	final Point size = this.tip.computeSize(SWT.DEFAULT, SWT.DEFAULT);
	final Point locn = this.getItemLocation(item);
	this.tip.setBounds(locn.x, locn.y, size.x, size.y);
	// klugey way to pass the Item to the label-listener
	this.label.setData("_TABLEITEM", item); //$NON-NLS-1$
	this.label.addListener(SWT.MouseExit, this.labelListener);
	this.label.addListener(SWT.MouseDown, this.labelListener);
	this.tip.setVisible(true);
    }

    /**
     * Dispose the tooltip
     */
    protected void disposeTip() {

	if (this.tip != null && !this.tip.isDisposed()) {
	    this.tip.dispose();
	    this.tip = null; // NOPMD
	    this.label = null; // NOPMD
	}
    }

    /**
     * Compute the location of the given Item in coordinates relative to display
     *
     * @param item
     *            Item
     * @return Point
     */
    protected abstract Point getItemLocation(final Item item);
    // Example:
    // Rectangle rect = ((TreeItem) item).getBounds(0)
    // return tree.toDisplay(rect.x, rect.y)

    /**
     * Retrieve the Item at the location of the Event
     *
     * @param event
     *            Event
     * @return Item
     */
    protected abstract Item retrieveItem(final Event event);
    // Example:
    // return tree.getItem(new Point(event.x, event.y))

    /**
     * Select the given Item as if the underlying Composite received the mouse
     * down event
     *
     * @param item
     *            Item
     */
    protected abstract void selectItem(final Item item);
    // Example:
    // tree.setSelection(new TreeItem[] { (TreeItem) item })
}
