/*******************************************************************************
 * Copyright 2018 fmCreation
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bcee.assistant;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateInfo;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.hbci.gui.input.BICInput;
import de.willuhn.jameica.hbci.gui.input.IBANInput;

/**
 * Dialog zum Abfragen der MahoeMain Kreditkarteninformationen
 *
 * @author Frank Mnster
 */
public class BceeCreateAccountDialog extends AbstractCreateAccountDialog<BceeCreateInfo> {

    /**
     * LuxTrustId
     */
    private transient TextInput luxTrustId;

    /**
     * IBAN
     */
    private transient IBANInput iban;

    /**
     * BIC
     */
    private transient BICInput bic;

    /**
     */
    public BceeCreateAccountDialog() {
        super();
        this.setTitle(I18N.tr("BceeCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

        super.paint(parent);

        this.luxTrustId = new TextInput(StringUtils.EMPTY, 20, StringUtils.EMPTY);
        this.luxTrustId.setMandatory(true);
        this.bic = new BICInput("BCEELULLXXX"); //$NON-NLS-1$
        this.bic.setEnabled(false);
        this.iban = new IBANInput(StringUtils.EMPTY, this.bic);
        this.iban.setMandatory(true);
        this.accDescription.setHint("BCEE-Spuerkeess"); //$NON-NLS-1$

        this.container.addLabelPair(I18N.tr("BceeCreateAccountDialog.iban"), this.iban); //$NON-NLS-1$
        this.container.addLabelPair(I18N.tr("BceeCreateAccountDialog.bic"), this.bic); //$NON-NLS-1$
        this.container.addLabelPair(I18N.tr("BceeCreateAccountDialog.luxTrustId"), this.luxTrustId); //$NON-NLS-1$

    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected BceeCreateInfo getData() {

        final CreateInfo info = super.getData();

        this.data.setOwner(info.getOwner());
        this.data.setComment(info.getComment());
        this.data.setDescription(StringUtils.isEmpty(info.getDescription()) ? "BCEE - Spuerkeess" //$NON-NLS-1$
                : info.getDescription());

        return this.data;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog#prepareData()
     */
    @Override
    protected boolean prepareData() {

        this.data = new BceeCreateInfo();

        this.data.setIban((String) this.iban.getValue());
        this.data.setBic((String) this.bic.getValue());
        this.data.setLuxTrustId((String) this.luxTrustId.getValue());
        this.data.setSavePDF(false);
        this.data.setSaveFolder(StringUtils.EMPTY);

        // TODO DatenCheck einbauen
        return true;
    }
}
