/*******************************************************************************
 * Copyright 2018 fmCreation
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bcee.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.List;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebAccountInformation;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeBankMessages;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.utils.TransactionSaver;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Job der die eigentliche Arbeit macht.
 * 
 * @author Frank Mnster
 * 
 */
public class BceeSynchronizeJobKontoauszug extends
		AbstractMahoeSynchronizeJobKontoauszug<BceeWebAccountInformation, BceeBankMessages> {

	/**
	 * Standartkonstruktor
	 */
	public BceeSynchronizeJobKontoauszug() {
		super();
	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
	 */
	@Override
	protected void getWebSite() throws WebSiteException, ApplicationException {
		this.webSite = BceeWebSite.getInstance(this.konto);
	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#login()
	 */
	@Override
	protected void login() throws WebSiteException, ApplicationException {

		final char[] password = PasswdCache.getPassword(Bcee.NAME, this.kundennummer);
		try {

			final String otp = BceeSynchronizeJobKontoauszugUtils.getOtp();
			this.webSite.login(password, otp);
		} catch (WebSiteException e) {
			if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
				// Passwort zurcksetzten
				PasswdCache.revoke(Bcee.NAME, this.kundennummer);
			}
			throw e;
		}
		Arrays.fill(password, ' ');

	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
	 */
	@Override
	protected void handleTransactions() throws WebSiteException, ApplicationException {

		// Vorgemerkte Umstze
		this.monitor.setStatusText(I18N.tr(
				"BceeSynchronizeJobKontoauszug.getPreBooked", this.accountInfo.getLastTransactionDateAsString())); //$NON-NLS-1$
		final List<Umsatz> preBooked = this.webSite.getPreBookedTransactions();
		// Debug-Ausgabe
		if (Logger.isLogging(Level.DEBUG)) {
			for (final Umsatz umsatz : preBooked) {

				String umsatzString;
				try {
					umsatzString = I18N.tr(
							"SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
							umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
							Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
							umsatz.getKommentar());
					Logger.debug(umsatzString);
				} catch (RemoteException e) {
					Logger.error("Fehler beim Erstellen des Umsatzinfostrings.", e); //$NON-NLS-1$
				}
			}
		}
		// Speicher vorgemerkte Umstze
		this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.savingPreBooked")); //$NON-NLS-1$
		final int preBookedCount = TransactionSaver.savePreBooked(this.konto, preBooked, false);
		this.monitor.setStatusText(I18N.tr(
				"BceeSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
		this.monitor.setPercentComplete(40);
		Logger.info(I18N.tr("BceeSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$

		// Gebuchte Umstze
		this.monitor.setStatusText(I18N.tr(
				"SynchronizeJobKontoauszug.getBooked", this.accountInfo.getLastTransactionDateAsString())); //$NON-NLS-1$
		final List<Umsatz> booked = this.webSite.getBookedTransactions();
		// Debug-Ausgabe
		if (Logger.isLogging(Level.DEBUG)) {
			for (final Umsatz umsatz : booked) {

				String umsatzString;
				try {
					umsatzString = I18N.tr(
							"SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
							umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
							Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
							umsatz.getKommentar());
					Logger.debug(umsatzString);
				} catch (RemoteException e) {
					Logger.error("Fehler beim Erstellen des Umsatzinfostrings.", e); //$NON-NLS-1$
				}
			}
		}

		// Speicher gebuchte Umstze
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
		final int bookedCount = TransactionSaver.saveBooked(this.konto, booked);
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
		this.monitor.setPercentComplete(50);
		Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

		// Saldo/verfgbarer Betrag setzten
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.updateSaldo")); //$NON-NLS-1$
		TransactionSaver.saveSaldo(this.konto, this.accountInfo.getSaldo());
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.newSaldoIs", //$NON-NLS-1$
				Double.toString(this.accountInfo.getSaldo())));
		if (this.accountInfo.getCreditLimit() == null) {
			TransactionSaver.saveSaldoAvailable(this.konto, 0.0d);
			this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.creditLimitMissing")); //$NON-NLS-1$
		} else {
			final double available = Math.max(0.0d, this.accountInfo.getSaldo() + this.accountInfo.getCreditLimit());
			TransactionSaver.saveSaldoAvailable(this.konto, available);
			this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.newAvailable", //$NON-NLS-1$
					Double.toString(available)));
			this.monitor.setPercentComplete(55);
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#handleAccountSheets()
	 */
	@Override
	protected void handleAccountSheets() throws WebSiteException, ApplicationException {
		// Untersttzt mein Konto nicht
		this.monitor.setStatusText(I18N.tr("BceeSynchronizeJobKontoauszug.notImplemented")); //$NON-NLS-1$
	}

}
