/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bcee.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeJobProvider;
import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.utils.BankToAdd;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung des Job-Providers fr das Einreichen einer neuen SEPA
 * berweisung der BCEE.
 *
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
@BankToAdd(blz = Bcee.BLZ, name = Bcee.NAME)
public class BceeSynchronizeJobProviderSepaUeberweisung implements MahoeSynchronizeJobProvider {

    /**
     * Liste der von diesem Backend implementierten Jobs
     */
    private final static List<Class<? extends SynchronizeJob>> JOBS = new ArrayList<>();

    static {
	{
	    BceeSynchronizeJobProviderSepaUeberweisung.JOBS.add(BceeSynchronizeJobSepaUeberweisung.class);
	}
    }
    /**
     * Die vom Job-Provider explizit unterstuetzten Konto-Arten. Alle anderen
     * machen andere Provider.
     */
    private final static Set<KontoType> SUPPORTED = new HashSet<>();

    static {
	{
	    BceeSynchronizeJobProviderSepaUeberweisung.SUPPORTED.add(KontoType.GIRO);
	}
    }

    @Resource
    private final MahoeSynchronizeBackend backend = null; // NOPMD

    /**
     * Standardkonstruktor
     */
    public BceeSynchronizeJobProviderSepaUeberweisung() {
	// Absichtlich leer
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final Object arg0) {

	// Nach Moeglichkeit zuerst
	return -1;
    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getJobTypes()
     */
    @Override
    public List<Class<? extends SynchronizeJob>> getJobTypes() {

	return BceeSynchronizeJobProviderSepaUeberweisung.JOBS;
    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getSynchronizeJobs(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public List<SynchronizeJob> getSynchronizeJobs(final Konto konto) {

	final Class<SynchronizeJobSepaUeberweisung> type = SynchronizeJobSepaUeberweisung.class;
	final List<SynchronizeJob> jobs = new LinkedList<>();
	for (final Konto kto : this.backend.getSynchronizeKonten(konto)) {
	    try {
		// Checken, ob das ein MahoeMain-Konto ist
		if (!this.backend.supports(type, konto)) {
		    continue;
		}

		// Supporten wir das Konto
		if (!this.supports(type, konto)) {
		    continue;
		}

		final SynchronizeOptions options = new SynchronizeOptions(kto); // NOPMD

		// Sync-Option fr SEPA-berweisungen aktiv?
		if (!options.getSyncAuslandsUeberweisungen()) {
		    continue;
		}

		// Now add the open transactions to teh job list
		addUeberweisungen(kto, jobs);

	    } catch (RemoteException | ApplicationException e) {
		Logger.error(I18N.tr("BceeSynchronizeJobProviderSepaUeberweisung.getSynchnizedJobsFailed"), e); //$NON-NLS-1$
	    }
	}

	return jobs;

    }

    /**
     * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#supports(java.lang.Class,
     *      de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public boolean supports(final Class<? extends SynchronizeJob> type, final Konto konto) {

	boolean support = false;

	// Kein Konto angegeben. Dann gehen wir mal davon aus, dass es geht
	if (konto == null) {
	    support = true;

	} else {
	    KontoType kontoType = null;
	    try {

		// Checken, ob es die BCEE-Konto ist, aber via BIC, nicht BLZ
		if (Bcee.BIC.equals(konto.getBic())) {

		    // Checken, ob das vielleicht ein nicht unterstuetztes Konto
		    // ist
		    kontoType = KontoType.find(konto.getAccountType());

		    // Wenn kein konkreter Typ angegeben ist, dann unterstuetzen
		    // wir
		    // es
		    if (kontoType == null || BceeSynchronizeJobProviderSepaUeberweisung.SUPPORTED.contains(kontoType)) {
			support = true;
		    }
		}
	    } catch (final RemoteException re) {
		Logger.error(I18N.tr("BceeSynchronizeJobProviderKontoauszug.determineSupportForAccountTypeFailed") //$NON-NLS-1$
			+ kontoType, re);
	    }
	}
	return support;
    }

    /**
     * Adds open transactions to the job list
     *
     * @param kto
     *            account to search for open transactions
     * @param jobs
     *            list to add the transaction jobs
     * @throws RemoteException
     * @throws ApplicationException
     */
    private void addUeberweisungen(final Konto kto, final List<SynchronizeJob> jobs)
	    throws RemoteException, ApplicationException {
	// Einzelueberweisungen Jobliste erstellen
	final DBIterator<?> list = kto.getAuslandsUeberweisungen();

	// Schnelleres Laden durch vorheriges Aussortieren
	list.addFilter("(ausgefuehrt is null or ausgefuehrt = 0)"); //$NON-NLS-1$
	while (list.hasNext()) {
	    final AuslandsUeberweisung sepaUeberweisung = (AuslandsUeberweisung) list.next();
	    if (!sepaUeberweisung.ueberfaellig() || sepaUeberweisung.ausgefuehrt()) {
		continue; // Nur ueberfaellige Auftraege
	    }

	    final SynchronizeJobSepaUeberweisung job = this.backend.create(SynchronizeJobSepaUeberweisung.class, kto);
	    job.setContext(SynchronizeJob.CTX_ENTITY, sepaUeberweisung);
	    jobs.add(job);
	}
    }

}
