/*******************************************************************************
 * Copyright 2018 fmCreation
 *  
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bcee.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobSepaUeberweisung;
import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeBankMessages;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebAccountInformation;
import de.frankmuenster.hibiscus.mahoe.bcee.web.BceeWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Job der die eigentliche Arbeit macht.
 *
 * @author Frank Mnster
 */
public class BceeSynchronizeJobSepaUeberweisung
        extends AbstractMahoeSynchronizeJobSepaUeberweisung<BceeWebAccountInformation, BceeBankMessages> {

    /**
     * Standartkonstruktor
     */
    public BceeSynchronizeJobSepaUeberweisung() {
        super();
    }

    /**
     * @see BceeSynchronizeJobSepaUeberweisung#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = BceeWebSite.getInstance(this.konto);
    }

    /**
     * @see BceeSynchronizeJobSepaUeberweisung#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        final char[] password = PasswdCache.getPassword(Bcee.NAME, this.kundennummer);
        try {

            final String otp = BceeSynchronizeJobKontoauszugUtils.getOtp();
            this.webSite.login(password, otp);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
                // Passwort zurcksetzten
                PasswdCache.revoke(Bcee.NAME, this.kundennummer);
            }
            throw e;
        }
        Arrays.fill(password, ' ');

    }

    /**
     * @see BceeSynchronizeJobSepaUeberweisung#handleGiroTransfer()
     */
    @Override
    protected void handleGiroTransfer() throws WebSiteException, ApplicationException {

        try {
            this.webSite.createSepaTransfer(this.aUeberweisung);
            try {
                final String resultMsg = this.aUeberweisung.getMeta("resultMsg", null); //$NON-NLS-1$
                if (resultMsg != null) {
                    this.monitor.setStatusText(resultMsg);
                }
            } catch (final RemoteException e) {
                // Wird nur geloggt
                Logger.error(I18N.tr("BceeSynchronizeJobSepaUeberweisung.readMetaDataFailed"), e); //$NON-NLS-1$
            }
            this.markExecuted();

        } catch (WebSiteException | ApplicationException e) {
            this.markFailed(e.getMessage());
            throw e;
        }

    }

}
