/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bsh.assistant;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateInfo;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Account;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.hbci.gui.input.BICInput;
import de.willuhn.jameica.hbci.rmi.KontoType;

/**
 * Dialog zum Abfragen der MahoeMain Kreditkarteninformationen
 *
 * @author Frank Mnster
 */
public class BshCreateAccountDialog extends AbstractCreateAccountDialog<BshCreateInfo> {

    /**
     * Liste der vorhandenen Vertrge
     */
    private final transient List<Account> accounts;

    /**
     * Select Input fr die Accounts
     */
    private transient SelectInput accountInput;

    /**
     * Kundennummer
     */
    private final transient String kdNummer;

    /**
     * Kundennummer Input
     */
    private transient TextInput kdNummerInput;

    /**
     * Erstellt den BSH Konto-Anlage Dialog
     *
     * @param kdNummer
     * @param accounts
     */
    public BshCreateAccountDialog(final String kdNummer, final List<Account> accounts) {

        super();
        this.setTitle(I18N.tr("BshCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
        this.kdNummer = kdNummer;
        this.accounts = accounts;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected BshCreateInfo getData() {

        final CreateInfo info = super.getData();

        this.data.setOwner(info.getOwner());
        this.data.setComment(info.getComment());
        this.data.setAccountType(KontoType.BAUSPAR);
        this.data.setDescription(StringUtils.isEmpty(info.getDescription()) ? "Schwbisch Hall" //$NON-NLS-1$
                : info.getDescription());

        return this.data;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

        super.paint(parent);

        this.kdNummerInput = new TextInput(this.kdNummer, 20, StringUtils.EMPTY);
        this.kdNummerInput.setEnabled(false);
        final BICInput bic = new BICInput(SchwaebischHall.BIC); // $NON-NLS-1$
        bic.setEnabled(false);

        this.container.addLabelPair(I18N.tr("BshCreateAccountDialog.kdNumber"), this.kdNummerInput); //$NON-NLS-1$
        this.container.addLabelPair(I18N.tr("BshCreateAccountDialog.bic"), bic); //$NON-NLS-1$

        this.accountInput = new SelectInput(this.accounts, this.accounts.get(0));
        this.accountInput.setAttribute("bsnType"); //$NON-NLS-1$
        this.container.addLabelPair(I18N.tr("BshCreateAccountDialog.existingAccounts"), this.accountInput); //$NON-NLS-1$

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog#prepareData()
     */
    @Override
    protected boolean prepareData() {

        this.data = new BshCreateInfo();

        this.data.setKdNummer((String) this.kdNummerInput.getValue());
        final Account choosen = (Account) this.accountInput.getValue();
        this.data.setAccount(choosen);

        // TODO DatenCheck einbauen
        return true;

    }

}
