
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * Vertragdetails - Daten
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "contracttype", "contractphase", "coredata", "subsidy", "debit", "contract", "assignmentgoal",
	"transactions" })
public class AccountDetailData {

    /**
     * Zuteilungsaussicht
     */
    @JsonProperty("assignmentgoal")
    private AssignmentGoal assignmentGoal;

    /**
     * Vertragsdaten
     */
    @JsonProperty("contract")
    private Contract contract;

    /**
     * Vertragsphase
     */
    @JsonProperty("contractphase")
    private Integer contractPhase;

    /**
     * Vertragstyp
     */
    @JsonProperty("contracttype")
    private Integer contractType;

    /**
     * Vertrags Coredaten
     */
    @JsonProperty("coredata")
    private CoreData coreData;

    /**
     * Einzge
     */
    @JsonProperty("debit")
    private Debit debit;

    /**
     * Staatliche F&ouml;rderung
     */
    @JsonProperty("subsidy")
    private Subsidy subsidy;

    /**
     * Ums&auml;tze
     */
    @JsonProperty("transactions")
    private List<TransactionYear> transactionYear;

    /**
     * Standardkonstruktor
     */
    public AccountDetailData() {
	// Absichtlich leer
    }

    /**
     * Liefert die unverbindliche Zuteilungsaussicht
     *
     * @return die unverbindliche Zuteilungsaussicht
     */
    @JsonProperty("assignmentgoal")
    public AssignmentGoal getAssignmentgoal() {

	return this.assignmentGoal;
    }

    /**
     * Liefert die Vertagsdaten
     *
     * @return die Vertagsdaten
     */
    @JsonProperty("contract")
    public Contract getContract() {

	return this.contract;
    }

    /**
     * Liefert die Vertragsphase
     *
     * @return die Vertragsphase
     */
    @JsonProperty("contractphase")
    public Integer getContractphase() {

	return this.contractPhase;
    }

    /**
     * Liefert den Vertragstyp
     *
     * @return den Vertragstyp
     */
    @JsonProperty("contracttype")
    public Integer getContracttype() {

	return this.contractType;
    }

    /**
     * Liefert die Vertrags Coredaten
     *
     * @return die Vertrags Coredaten
     */
    @JsonProperty("coredata")
    public CoreData getCoreData() {

	return this.coreData;
    }

    /**
     * Liefert die Einzge
     *
     * @return die Einzge
     */
    @JsonProperty("debit")
    public Debit getDebit() {

	return this.debit;
    }

    /**
     * Liefert die staatliche F&ouml;rderung
     *
     * @return die staatliche F&ouml;rderung
     */
    @JsonProperty("subsidy")
    public Subsidy getSubsidy() {

	return this.subsidy;
    }

    /**
     * Liefert die Ums&auml;tze
     *
     * @return eine Liste mit den Ums&auml;tze
     */
    @JsonProperty("transactions")
    public List<TransactionYear> getTransactions() {

	return this.transactionYear;
    }

}
