
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;

/**
 * Hilfsklasse die eine Umsatzliste aus den Vertragsdetails erstellt.
 *
 * @author Frank Mnster
 */
public final class AccountDetailMapper {

    /**
     * Die Vertragdetails mit dem Umstzen der letzten 3 Jahre
     */
    private AccountDetailData data;

    /**
     * Erzeugt einen AccountMapper fr die AccountDate
     *
     * @param data
     */
    public AccountDetailMapper(final AccountDetailData data) {
	this.data = data;
    }

    /**
     * @return the data
     */
    public AccountDetailData getData() {

	return this.data;
    }

    /**
     * Liefert die Umsatzliste aus den Vertragdetails
     *
     * @return die Umsatzliste
     */
    public List<Umsatz> getUmsatzList() {

	final List<Umsatz> umsaetze = new ArrayList<>();

	// loop over all years
	for (final TransactionYear transactionYear : this.data.getTransactions()) {

	    // get year balance
	    BigDecimal saldo = transactionYear.getSaldo();

	    // loop over month
	    for (final TransactionMonth transactionMonth : transactionYear.getMonth()) {
		// get all transactions from a month
		for (final TransactionEntry transactionEntry : transactionMonth.getEntries()) {
		    // add entry
		    this.addEntry(umsaetze, transactionEntry, saldo);
		    // if entry has an amount subtract from year balance to have
		    // the correct balance for the next entry
		    if (transactionEntry.getAmount() != null) {
			saldo = saldo.subtract(transactionEntry.getAmount());
		    }
		}
	    }
	}

	return umsaetze;

    }

    /**
     * @param data
     *            the data to set
     */
    public void setData(final AccountDetailData data) {

	this.data = data;
    }

    private void addEntry(final List<Umsatz> umsaetze, final TransactionEntry transactionEntry,
	    final BigDecimal saldo) {

	try {
	    // Neuen Umsatz anlegen
	    final HBCIDBService dbService = Settings.getDBService();
	    final Umsatz umsatz = dbService.createObject(Umsatz.class, null);

	    // Datum
	    umsatz.setDatum(transactionEntry.getDate());
	    umsatz.setValuta(transactionEntry.getDate());

	    // Zweck
	    Utils.loadUmsatz(umsatz, transactionEntry.getSubject());

	    // Betrag
	    umsatz.setBetrag(transactionEntry.getAmount().doubleValue());

	    // Saldo
	    umsatz.setSaldo(saldo.doubleValue());

	    // Zur Liste hinzufgen
	    umsaetze.add(umsatz);

	} catch (final RemoteException re) {
	    Logger.error(I18N.tr("AccountDetailMapper.createUmsatzFailed", re.getMessage()), re); //$NON-NLS-1$
	}

    }

}
