
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;
import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * JSON Vertrags Objekt
 *
 * @author Frank Mnster
 */
@JsonIgnoreProperties(ignoreUnknown = true)
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "tarif", "savesum", "startDate", "chooseassignment", "moreassignment", "bonusamount",
	"currentbalance", "savinginterestrate", "ratingnumber", "grantinterestamount", "grantremaining", "grantend" })
public class Contract {

    /**
     * Der Bonus-Bbetrag
     */
    @JsonProperty("bonusamount")
    private String bonusAmount;

    /**
     * Wahl-, Mehrzuteilung in %
     */
    @JsonProperty("chooseassignment")
    private String chooseAssignment;

    /**
     * Lfd. Saldo
     */
    @JsonProperty("currentbalance")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal currentBalance;

    /**
     * Weitere Zuteilungen ???
     */
    @JsonProperty("moreassignment")
    private String moreAssignment;

    /**
     * Bewertungszahl
     */
    @JsonProperty("ratingnumber")
    private String ratingNumber;

    /**
     * Bausparsumme
     */
    @JsonProperty("savesum")
    @JsonDeserialize(using = SimpleMoneyDeserializer.class)
    private BigDecimal saveSum;

    /**
     * Sparzins
     */
    @JsonProperty("savinginterestrate")
    private String savingInterestRate; // NOPMD

    /**
     * Vertragsbeginn
     */
    @JsonProperty("startdate")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
    private Date startDate;

    /**
     * Tarif
     */
    @JsonProperty("tarif")
    private String tarif;

    /**
     * H&ouml;he der Rate
     */
    @JsonProperty("grantinterestamount")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal grantInterestAmount; // NOPMD

    /**
     * Restschuld
     */
    @JsonProperty("grantremaining")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal grantRemaining; // NOPMD

    /**
     * Voraussichtliches Vertragsende
     */
    @JsonProperty("grantend")
    private String grantend;

    /**
     * Standardkonstruktor
     */
    public Contract() {
	// Absichtlich leer
    }

    /**
     * Liefert den Bouns-Betrag
     *
     * @return den Bouns-Betrag
     */
    @JsonProperty("bonusamount")
    public String getBonusamount() {

	return this.bonusAmount;
    }

    /**
     * Liefert die Wahl-, Mehrzuteilung in %
     *
     * @return die Wahl-, Mehrzuteilung in %
     */
    @JsonProperty("chooseassignment")
    public String getChooseassignment() {

	return this.chooseAssignment;
    }

    /**
     * Liefert das lfd. Saldo
     *
     * @return das lfd. Saldo
     */
    @JsonProperty("currentbalance")
    public BigDecimal getCurrentbalance() {

	return this.currentBalance;
    }

    /**
     * Liefert das voraussichtliches Vertragsende
     *
     * @return das voraussichtliches Vertragsende
     */
    public String getGrantend() {

	return this.grantend;
    }

    /**
     * Liefert die H&ouml;he der Rate
     *
     * @return die H&ouml;he der Rate
     */

    public BigDecimal getGrantInterestAmount() {

	return this.grantInterestAmount;
    }

    /**
     * Liefert die Restschuld
     *
     * @return die Restschuld
     */

    public BigDecimal getGrantRemaining() {

	return this.grantRemaining;
    }

    /**
     * Liefert das lfd. Saldo.
     *
     * @return das lfd. Saldo.
     */
    @JsonProperty("moreassignment")
    public String getMoreAssignment() {

	return this.moreAssignment;
    }

    /**
     * Liefert die Bewertungszahl.
     *
     * @return die Bewertungszahl.
     */
    @JsonProperty("ratingnumber")
    public String getRatingNumber() {

	return this.ratingNumber;
    }

    /**
     * Liefert die Bausparsumme
     *
     * @return die Bausparsumme
     */

    public BigDecimal getSaveSum() {

	return this.saveSum;
    }

    /**
     * Liefert den Sparzins
     *
     * @return den Sparzins
     */
    @JsonProperty("savinginterestrate")
    public String getSavingInterestRate() {

	return this.savingInterestRate;
    }

    /**
     * Liefert den Vertragsbeginn
     *
     * @return den Vertragsbeginn
     */
    @JsonProperty("startdate")
    public Date getStartDate() {

	return this.startDate;
    }

    /**
     * Liefert den Tarif
     *
     * @return den Tarif
     */
    public String getTarif() {

	return this.tarif;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {

	return String.format(
		"Contract [bonusAmount=%s, chooseAssignment=%s, currentBalance=%s, moreAssignment=%s, ratingNumber=%s, saveSum=%s, savingInterestRate=%s, startDate=%s, tarif=%s]", //$NON-NLS-1$
		this.bonusAmount, this.chooseAssignment, this.currentBalance, this.moreAssignment, this.ratingNumber,
		this.saveSum, this.savingInterestRate, this.startDate, this.tarif);
    }
}
