
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;

/**
 * Vertragsdetails - Debit (Einzug)
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "suggestedamount", "showDirectDebitLink", "directdebits" })
public class Debit {

    /**
     * Der Regelsparbetrag
     */
    @JsonProperty("suggestedamount")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal suggestedAmount;

    /**
     * Den "Bankeinzug bearbeiten" Link anzeigen
     */
    @JsonProperty("showDirectDebitLink")
    private Boolean showDirectDebitLink; // NOPMD

    /**
     * Liste mit allen direkten Einzgen
     */
    @JsonProperty("directdebits")
    private List<DirectDebit> directDebits;

    /**
     * Standardkonstruktor
     */
    public Debit() {
	// Absichtlich leer
    }

    /**
     * Liefert die Liste mit allen direkten Einzgen
     *
     * @return die Liste mit allen direkten Einzgen
     */
    @JsonProperty("directdebits")
    public List<DirectDebit> getDirectDebits() {

	return this.directDebits;
    }

    /**
     * Liefert den Regelsparbetrag
     *
     * @return den Regelsparbetrag
     */
    @JsonProperty("suggestedamount")
    public BigDecimal getSuggestedAmount() {

	return this.suggestedAmount;
    }

    /**
     * Liefert die Einstellung, ob der "Bankeinzug bearbeiten" Link anzeigen
     * werden soll oder nicht
     *
     * @return "Bankeinzug bearbeiten" Link anzeigen
     */
    @JsonProperty("showDirectDebitLink")
    public Boolean isShowDirectDebitLink() {

	return this.showDirectDebitLink;
    }

}
