
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * Vertragsdetails - Directdebit (Einzug)
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "owner", "iban", "bic", "bank", "interval", "amount", "startdate", "enddate" })
public class DirectDebit {

    /**
     * Einzugsbetrag
     */
    @JsonProperty("amount")
    @JsonDeserialize(using = SimpleMoneyDeserializer.class)
    private BigDecimal amount;

    /**
     * Die Bank
     */
    @JsonProperty("bank")
    private String bank;

    /**
     * BIC der Bank
     */
    @JsonProperty("bic")
    private String bic;

    /**
     * IBAN der Bank
     */
    @JsonProperty("iban")
    private String iban;

    /**
     * Einzugsturnus
     */
    @JsonProperty("interval")
    private String interval;

    /**
     * Kontoinhaber
     */
    @JsonProperty("owner")
    private String owner;

    /**
     * Anfangsdatum
     */
    @JsonProperty("startdate")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
    private Date startDate;

    /**
     * Enddatum
     */
    @JsonProperty("enddate")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
    private Date endDate;

    /**
     * Standardkonstruktor
     */
    public DirectDebit() {
	// Absichtlich leer
    }

    /**
     * Liefert den Einzugsbetrag
     *
     * @return den Einzugsbetrag
     */
    @JsonProperty("amount")
    public BigDecimal getAmount() {

	return this.amount;
    }

    /**
     * Liefert die Bank
     *
     * @return die Bank
     */
    @JsonProperty("bank")
    public String getBank() {

	return this.bank;
    }

    /**
     * Liefert die BIC
     *
     * @return die BIC
     */
    @JsonProperty("bic")
    public String getBic() {

	return this.bic;
    }

    /**
     * Liefert das Enddatum
     *
     * @return das Enddatum
     */
    @JsonProperty("enddate")
    public Date getEndDate() {

	return this.endDate;
    }

    /**
     * Liefert die IBAN
     *
     * @return die IBAN
     */
    @JsonProperty("iban")
    public String getIban() {

	return this.iban;
    }

    /**
     * Liefert den Einzugsturnus
     *
     * @return den Einzugsturnus
     */
    @JsonProperty("interval")
    public String getInterval() {

	return this.interval;
    }

    /**
     * Liefert den Kontoinhaber
     *
     * @return den Kontoinhaber
     */
    @JsonProperty("owner")
    public String getOwner() {

	return this.owner;
    }

    /**
     * Liefert das Startdatum
     *
     * @return das Startdatum
     */
    @JsonProperty("startdate")
    public Date getStartDate() {

	return this.startDate;
    }

}
