
package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;
import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.TransactionDateDeserializer;

/**
 * Vertargdetails - Umsatzeintrag
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "amount", "subject", "day" })
public class TransactionEntry {

    /**
     * Der Buchungsbetrag
     */
    @JsonProperty("amount")
    @JsonDeserialize(using = MoneyDeserializer.class)
    private BigDecimal amount;

    /**
     * Der Buchungstext
     */
    @JsonProperty("subject")
    private String subject;

    /**
     * Das Buchungsdatum
     */
    @JsonProperty("day")
    @JsonDeserialize(using = TransactionDateDeserializer.class)
    private Date date;

    /**
     * Standardkonstruktor
     */
    public TransactionEntry() {
	// Absichtlich leer
    }

    /**
     * Liefert den Betrag der Buchung.
     *
     * @return den Betrag der Buchung
     */
    @JsonProperty("amount")
    public BigDecimal getAmount() {

	return this.amount;
    }

    /**
     * Das Datum der Buchung
     *
     * @return das Datum der Buchung
     */
    @JsonProperty("day")
    public Date getDate() {

	return this.date;
    }

    /**
     * Der Tag der Buchung
     *
     * @return der Buchungstag
     */
    public Integer getDay() {

	final Calendar cal = Calendar.getInstance();
	cal.setTime(this.date);
	return cal.get(Calendar.DATE);
    }

    /**
     * Liefert den Buchungstext.
     *
     * @return der Buchungstext
     */
    @JsonProperty("subject")
    public String getSubject() {

	return this.subject;
    }

}
