
package de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;

import de.frankmuenster.hibiscus.mahoe.bsh.json.Message;

/**
 * Deserializer fr die Nachrichten-Liste. Im Prinzip hebt sie die Liste eine
 * Ebene hher und umgeht so das "data" JSON Object
 *
 * @author Frank Mnster
 */
public class MessagesDeserializer extends JsonDeserializer<List<Message>> {

    /**
     * Contracts JSON identifier
     */
    private static final String MESSAGES = "messages"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public MessagesDeserializer() {
	super();
    }

    /**
     * @see com.fasterxml.jackson.databind.JsonDeserializer#deserialize(com.fasterxml.jackson.core.JsonParser,
     *      com.fasterxml.jackson.databind.DeserializationContext)
     */
    @Override
    public List<Message> deserialize(final JsonParser jsonParser, final DeserializationContext context)
	    throws IOException {

	List<Message> messagesList = new ArrayList<>();

	final ObjectCodec objectCodec = jsonParser.getCodec();
	final JsonNode jsonNode = objectCodec.readTree(jsonParser);

	final Iterator<Map.Entry<String, JsonNode>> fields = jsonNode.fields();
	while (fields.hasNext()) {
	    final Map.Entry<String, JsonNode> field = fields.next();
	    final String fieldName = field.getKey();
	    if (MessagesDeserializer.MESSAGES.equals(fieldName)) {
		final JsonNode fieldValue = field.getValue();
		if (!fieldValue.isObject()) {
		    final Message[] accounts = objectCodec.treeToValue(fieldValue, Message[].class);
		    messagesList = Arrays.asList(accounts);
		    break;
		}
	    }
	}

	return messagesList;
    }
}
