/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.DateTimeException;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonStreamContext;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.TransactionMonth;
import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.TransactionYear;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * Deserializer fr das Datum der BSH
 *
 * @author Frank Mnster
 */
public class TransactionDateDeserializer extends JsonDeserializer<Date> {

    /**
     * Datumsformatter for BSH with new DateTimeFormatter pattern (LLL is SHORT notation of month)
     */
    private static final DateTimeFormatter sdf = DateTimeFormatter.ofPattern("d.LLL.yyyy", Locale.GERMAN); //$NON-NLS-1$

    /**
     * Pattern fr den kurzen Monatsnamen im System
     */
    private static final String SHORT_MONTH_PAT = "MMM"; //$NON-NLS-1$

    /**
     * Default Monatsname, falls nicht angegeben
     */
    private static final String DEFAULT_MONTH = "Jan"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public TransactionDateDeserializer() {

        super();
    }

    /**
     * @see JsonDeserializer#deserialize(JsonParser, DeserializationContext)
     */
    @Override
    public Date deserialize(final JsonParser jsonParser, final DeserializationContext context) throws IOException {

        final SimpleDateFormat sdfMonth = new SimpleDateFormat(TransactionDateDeserializer.SHORT_MONTH_PAT,
                Locale.GERMAN);

        String month = TransactionDateDeserializer.DEFAULT_MONTH;
        int year = 2000;

        // Monat ermitteln
        final JsonStreamContext jsonMonth = jsonParser.getParsingContext().getParent().getParent();
        if (jsonMonth.getCurrentValue() instanceof TransactionMonth) {
            final TransactionMonth tMonth = (TransactionMonth) jsonMonth.getCurrentValue();
            month = tMonth.getName();
            if (StringUtils.isEmpty(month)) {
                month = TransactionDateDeserializer.DEFAULT_MONTH;
            }
            if ("Mrz".equalsIgnoreCase(month)) { //$NON-NLS-1$
                final Calendar cal = Calendar.getInstance();
                cal.set(Calendar.MONTH, 2);
                month = sdfMonth.format(cal.getTime());
            }
        }

        final JsonStreamContext jsonYear = jsonMonth.getParent().getParent();
        if (jsonYear.getCurrentValue() instanceof TransactionYear) {
            final TransactionYear tYear = (TransactionYear) jsonYear.getCurrentValue();
            year = tYear.getYear();
        }

        String day = jsonParser.getText();
        if (StringUtils.isEmpty(day)) {
            day = "1"; //$NON-NLS-1$
        }

        final StringBuilder textDate = new StringBuilder(10);
        textDate.append(day).append('.').append(month).append('.').append(year);

        Date parse;
        try {
            final LocalDate localDate = LocalDate.parse(textDate.toString(), TransactionDateDeserializer.sdf);
            parse = Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
        } catch (final DateTimeException e) {
            Logger.error(I18N.tr("TransactionDateDeserializer.parseDateFailed", textDate.toString()), e); //$NON-NLS-1$
            throw new JsonParseException(jsonParser,
                    I18N.tr("TransactionDateDeserializer.parsedateFailedExceptionText"), e); //$NON-NLS-1$
        }

        return parse;

    }
}
