
package de.frankmuenster.hibiscus.mahoe.bsh.web;

import java.util.Date;
import java.util.List;

import com.gargoylesoftware.htmlunit.WebClient;

import de.frankmuenster.hibiscus.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.hibiscus.mahoe.messages.MahoeMessage;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Bank-Nachrichten der BSH
 *
 * @author Frank Mnster
 */
public class BshBankMessages extends AbstractBankMessages {

    /**
     * Konstruktor
     * 
     * @param webClient
     */
    public BshBankMessages(final WebClient webClient) {
        super(webClient);
    }

    /**
     * @see AbstractBankMessages#readMessages(Date)
     */
    @Override
    public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

        if (BshWebSiteUtils.getNewMessageCount(this.webclient) > 0) {
            Logger.info("Lese neue Institutsnachrichten."); //$NON-NLS-1$

            final List<MahoeMessage> mesgs = BshWebSiteUtils.getNewMessages(this.webclient);
            this.messages.addAll(mesgs);

        }

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.hibiscus.mahoe.messages.MahoeMessage)
     */
    @Override
    public void markAsRead(final MahoeMessage message) throws ApplicationException {

        try {
            BshWebSiteUtils.markMessageAsRead(this.webclient, message);
        } catch (final WebSiteException e) {
            Logger.error(I18N.tr("BshBankMessages.markAsReadFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("BshBankMessages.markAsReadFailed"), e); //$NON-NLS-1$
        }

    }
}
