package de.frankmuenster.hibiscus.mahoe.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
import de.willuhn.jameica.hbci.gui.input.KontoInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public abstract class AbstractSettingsController extends AbstractControl implements SettingsController {

    /**
     * (Kreditkarten)-Konto
     */
    protected transient KontoInput account;

    /**
     * Die BLZ des Bank fr den Kontroller
     */
    private final transient String blz;

    /**
     * Standardkonstruktor
     *
     * @param blz
     */
    public AbstractSettingsController(final String blz) {
        super(null);
        this.blz = blz;
    }

    /**
     * Liefert ein Kontoauswahlfeld mit allen Konten dieser Bank.
     *
     * @return ein Kontoauswahlfeld mit allen Konten dieser Bank.
     */
    @Override
    public KontoInput getKonto() {

        if (this.account == null) {
            try {
                final KontoFilter ktoFilter = new KontoFilter() {

                    /**
                     * @see de.willuhn.jameica.hbci.gui.filter.KontoFilter#accept(Konto)
                     */
                    @Override
                    public boolean accept(final Konto konto) throws RemoteException {

                        // Muss auf jeden Fall erst mal ein Online-Konto sein.
                        if (!KontoFilter.ONLINE.accept(konto)) {
                            return false;
                        }
                        // Jetzt noch checken, ob die BLZ zur Santander passt
                        final String blz1 = konto.getBLZ();
                        return StringUtils.isNotEmpty(blz1) && blz1.equals(AbstractSettingsController.this.blz);
                    }
                };
                this.account = new KontoInput(null, ktoFilter);
                this.account.addListener(new Listener() {

                    /**
                     * @see Listener#handleEvent(Event)
                     */
                    @Override
                    public void handleEvent(final Event event) {

                        AbstractSettingsController.this.handleAccountChanged(event);
                    }
                });
            } catch (final RemoteException e) {
                final String msg = I18N.tr("SantanderSettingsController.createAccountListFailed"); //$NON-NLS-1$
                Logger.error(msg, e);
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
            }
            final Event event = new Event();
            event.type = SWT.Selection;
            this.handleAccountChanged(event);
        }
        return this.account;
    }

    /**
     * Behandelt einen Kontowechsel
     *
     * @param event
     */
    protected abstract void handleAccountChanged(final Event event);
}
