package de.frankmuenster.hibiscus.mahoe.gui.parts;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.gui.controller.AktienSettingsController;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * @author Frank Mnster
 */
public class AktienSettingsPart implements Part {

    /**
     * Unser Kontroller
     */
    private final transient AktienSettingsController control;

    /**
     * Erstellt ein SantanderSetting Part mit allen Einstellungsmglichkeiten
     * der Santander Bank
     *
     * @param control
     *            der SantanderSettingsController
     */
    public AktienSettingsPart(final AktienSettingsController control) {
	this.control = control;
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

	final ColumnLayout colMain = new ColumnLayout(parent, 1);
	final SimpleContainer header = new SimpleContainer(colMain.getComposite());
	header.addHeadline(I18N.tr("AktienSettingsPart.accountSettingsHeadLine")); //$NON-NLS-1$
	header.addLabelPair(I18N.tr("AktienSettingsPart.accountLbl"), this.control.getKonto()); //$NON-NLS-1$
	final ColumnLayout clContent = new ColumnLayout(parent, 2);
	final GridLayout lay = (GridLayout) clContent.getComposite().getLayout();
	lay.makeColumnsEqualWidth = false;
	final SimpleContainer left = new SimpleContainer(clContent.getComposite(), false, 2);
	final SimpleContainer right = new SimpleContainer(clContent.getComposite(), true, 1);
	final GridData gridData = (GridData) right.getComposite().getLayoutData();
	gridData.widthHint = 30;
	gridData.heightHint = 500;
	left.addHeadline(I18N.tr("AktienSettingsPart.addStocks")); //$NON-NLS-1$
	left.addPart(this.control.getSearchPart());
	left.addPart(this.control.getSelectExchangePart());
	left.addLabelPair(I18N.tr("AktienSettingsPart.amount"), this.control.getStockAmountInput()); //$NON-NLS-1$
	final ButtonArea buttonArea = new ButtonArea();
	buttonArea.addButton(this.control.getUpdateStock());
	buttonArea.addButton(this.control.getAddStock());
	left.addButtonArea(buttonArea);
	right.addHeadline(I18N.tr("AktienSettingsPart.existingStocks")); //$NON-NLS-1$
	right.addPart(this.control.getAktienTable());
	right.addText(StringUtils.EMPTY, false);
	right.addText(StringUtils.EMPTY, false);
	right.addText(StringUtils.EMPTY, false);
	right.addText(StringUtils.EMPTY, false);
	right.addText(StringUtils.EMPTY, false);
    }
}
