
package de.frankmuenster.hibiscus.mahoe.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.gui.controller.SantanderSettingsController;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * @author Frank Mnster
 */
public class SantanderSettingsPart implements Part {

    /**
     *
     */
    private static final String FIX_SPACE_TEXT = "         "; //$NON-NLS-1$

    /**
     * Unser Kontroller
     */
    private final transient SantanderSettingsController control;

    /**
     * Erstellt ein SantanderSetting Part mit allen Einstellungsmglichkeiten
     * der Santander Bank
     *
     * @param control
     *            der SantanderSettingsController
     */
    public SantanderSettingsPart(final SantanderSettingsController control) {
        this.control = control;
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        final ColumnLayout colMain = new ColumnLayout(parent, 2);

        final SimpleContainer header = new SimpleContainer(colMain.getComposite());
        header.addHeadline(I18N.tr("SantanderSettingsPart.accountSettingsHeadLine")); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("SantanderSettingsPart.accountLbl"), this.control.getKonto()); //$NON-NLS-1$

        final ColumnLayout clContent = new ColumnLayout(parent, 2);
        final SimpleContainer left = new SimpleContainer(clContent.getComposite(), true, 1);
        final SimpleContainer right = new SimpleContainer(clContent.getComposite());

        final SimpleContainer left1 = new SimpleContainer(left.getComposite(), false, 4);
        // left1.addLabelPair(I18N.tr("SantanderSettingsPart.bankLbl"),
        // this.control.getBankTyp()); //$NON-NLS-1$
        left1.addLabelPair(I18N.tr("SantanderSettingsPart.accountTypLbl"), this.control.getKontoTyp()); //$NON-NLS-1$
        left1.addText(SantanderSettingsPart.FIX_SPACE_TEXT, false);
        left1.addText(SantanderSettingsPart.FIX_SPACE_TEXT, false);
        left1.addText(SantanderSettingsPart.FIX_SPACE_TEXT, false);

        final SimpleContainer left2a = new SimpleContainer(left.getComposite());
        left2a.addHeadline(I18N.tr("SantanderSettingsPart.accoutSheetsHeadLine")); //$NON-NLS-1$

        final SimpleContainer left2 = new SimpleContainer(left.getComposite(), false, 6);
        left2.addLabelPair(I18N.tr("SantanderSettingsPart.saveAccountSheetsLbl"), this.control.getSavePDF()); //$NON-NLS-1$
        left2.addLabelPair(I18N.tr("SantanderSettingsPart.saveFolderLbl"), this.control.getSavePdfFolder()); //$NON-NLS-1$
        left2.addText("                                ", false); //$NON-NLS-1$
        left2.addLabelPair(I18N.tr("SantanderSettingsPart.alwaysAskFileSave"), this.control.getAlwaysAskFileName()); //$NON-NLS-1$
        left2.addLabelPair(I18N.tr("SantanderSettingsPart.maxCountFileAskDownload"), this.control.getMaxAskCount()); //$NON-NLS-1$
        final SimpleContainer left3 = new SimpleContainer(left.getComposite());

        left3.addText(SantanderSettingsPart.FIX_SPACE_TEXT, true);

        left3.addHeadline(I18N.tr("SantanderSettingsPart.onlyForCreditCardHeadLine")); //$NON-NLS-1$
        left3.addLabelPair(I18N.tr("SantanderSettingsPart.cardNumberLbl"), this.control.getCardNumber()); //$NON-NLS-1$
        left3.addLabelPair(I18N.tr("SantanderSettingsPart.createClearingLbl"), this.control.getCreateClearing()); //$NON-NLS-1$
        left3.addLabelPair(I18N.tr("SantanderSettingsPart.clearingAccountLbl"), this.control.getClearingAccount()); //$NON-NLS-1$

        left.addText(SantanderSettingsPart.FIX_SPACE_TEXT, true);

        right.addText(SantanderSettingsPart.FIX_SPACE_TEXT, false);

    }

}
