
package de.frankmuenster.hibiscus.mahoe.messages;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.messaging.ImportMessage;
import de.willuhn.jameica.hbci.rmi.Nachricht;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * Eine Mahoe-Bank-Nachricht. Dient als Zwischenspeicher von Bank-Nachrichten,
 * bis sie in der Datenbank gespeichert werden.
 *
 * @author Frank Mnster
 */
public class MahoeMessage {

    /**
     * BLZ
     */
    private String blz;

    /**
     * Datum
     */
    private Date date;

    /**
     * ID
     */
    private String id; // NOPMD

    /**
     * Nachricht
     */
    private String message;

    /**
     * Betreff
     */
    private String subject;

    /**
     * Erzeugt eine neue MahoeNachricht
     *
     * @param blz
     * @param date
     * @param subject
     * @param message
     */
    public MahoeMessage(final String blz, final Date date, final String subject, final String message) {
	this.blz = blz;
	this.date = date;
	this.subject = subject;
	this.message = message;
    }

    /**
     * @return the blz
     */
    public String getBlz() {

	return this.blz;
    }

    /**
     * @return the date
     */
    public Date getDate() {

	return this.date;
    }

    /**
     * @return the id
     */
    public String getId() {

	return this.id;
    }

    /**
     * @return the message
     */
    public String getMessage() {

	return this.message;
    }

    /**
     * @return the subject
     */
    public String getSubject() {

	return this.subject;
    }

    /**
     * Liefert ein Hibiscus Nachricht, ohne
     *
     * @throws RemoteException
     * @throws ApplicationException
     */
    public void saveAsNachricht() throws RemoteException, ApplicationException {

	final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm", Locale.GERMAN); //$NON-NLS-1$

	final Nachricht msg = (Nachricht) Settings.getDBService().createObject(Nachricht.class, null);
	msg.setBLZ(this.blz);
	msg.setNachricht(this.message);
	msg.setDatum(this.date);
	msg.store();

	// Speicher doubleKey
	final String doubleKey = sdf.format(this.date) + this.subject;
	msg.setMeta("doubleKey", doubleKey); //$NON-NLS-1$

	Application.getMessagingFactory().sendMessage(new ImportMessage(msg));

    }

    /**
     * @param blz
     *            the blz to set
     */
    public void setBlz(final String blz) {

	this.blz = blz;
    }

    /**
     * @param date
     *            the date to set
     */
    public void setDate(final Date date) {

	this.date = date;
    }

    /**
     * @param id
     *            the id to set
     */
    public void setId(final String id) { // NOPMD

	this.id = id;
    }

    /**
     * @param message
     *            the message to set
     */
    public void setMessage(final String message) {

	this.message = message;
    }

    /**
     * @param subject
     *            the subject to set
     */
    public void setSubject(final String subject) {

	this.subject = subject;
    }

}
