/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.pdfextractor;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.Map;

/**
 * Defines a first pages PDF account sheet. It extends <code>AbstractBankPDFPage</code> to add regions for general
 * account information like account number, owner and balance.
 *
 * @author Frank Mnster
 */
public abstract class AbstractBankPDFPageFirst extends AbstractBankPDFPage {

    /**
     * Standard constructor call super class.
     *
     * @throws IOException
     */
    public AbstractBankPDFPageFirst() throws IOException {
        super();
        this.setFirstPageRegions();
    }

    /**
     * Defines the regions in a Map. The region rectangle values must able be converted to PDF measures. Following key
     * needs to be defined in the {@code Map}:<br>
     * <ul>
     * <li>{@code AbstractBankPDFPage.KARTEN_KONTO}</li>
     * <li>{@code AbstractBankPDFPage.KARTEN_INHABER}</li>
     * <li>{@code AbstractBankPDFPage.RECHNUNGS_DATUM}</li>
     * <li>{@code AbstractBankPDFPage.SALDO}</li>
     * <li>{@code AbstractBankPDFPage.FAELLIG_BETRAG}</li>
     * <li>{@code AbstractBankPDFPage.FAELLIG_DATUM}</li>
     * <li>{@code AbstractBankPDFPage.FAELLIG_IBAN}</li>
     * <li>{@code AbstractBankPDFPage.FAELLIG_BIC}</li>
     * </ul>
     *
     * @return a {@code Map} with the regions defined
     */
    protected abstract Map<String, Rectangle> defineFirstPageRegions();

    /**
     * Setzt die Regionen auf der ersten Seite
     */
    private void setFirstPageRegions() {

        final Map<String, Rectangle> regions = this.defineFirstPageRegions();

        this.setRegion(AbstractBankPDFPage.KARTEN_KONTO, regions);
        this.setRegion(AbstractBankPDFPage.KARTEN_INHABER, regions);
        this.setRegion(AbstractBankPDFPage.RECHNUNGS_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.SALDO, regions);
        this.setRegion(AbstractBankPDFPage.FAELLIG_BETRAG, regions);
        this.setRegion(AbstractBankPDFPage.FAELLIG_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.FAELLIG_IBAN, regions);
        this.setRegion(AbstractBankPDFPage.FAELLIG_BIC, regions);

    }
}
