/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.pdfextractor;

import java.awt.Rectangle;

import org.apache.pdfbox.text.PDFTextStripperByArea;

/**
 * @author Frank Mnster
 */
public class PdfRegionRectangle {

    /**
     * left start point
     */
    private int areaX;

    /**
     * top start point
     */
    private int areaY;

    /**
     * width
     */
    private int areaW;

    /**
     * height
     */
    private int areaH;

    /**
     * Creates a new <code>PdfRegionRectangle</code> with the given x,y
     * position, width and height.
     *
     * @param regionX
     * @param regionY
     * @param regionW
     * @param regionH
     */
    public PdfRegionRectangle(final int regionX, final int regionY, final int regionW, final int regionH) {
        this.areaX = regionX;
        this.areaY = regionY;
        this.areaW = regionW;
        this.areaH = regionH;

    }

    /**
     * @return the areaX
     */
    public int getAreaX() {

        return this.areaX;
    }

    /**
     * @param areaX
     *            the areaX to set
     */
    public void setAreaX(final int areaX) {

        this.areaX = areaX;
    }

    /**
     * @return the areaY
     */
    public int getAreaY() {

        return this.areaY;
    }

    /**
     * @param areaY
     *            the areaY to set
     */
    public void setAreaY(final int areaY) {

        this.areaY = areaY;
    }

    /**
     * @return the areaW
     */
    public int getAreaW() {

        return this.areaW;
    }

    /**
     * @param areaW
     *            the areaW to set
     */
    public void setAreaW(final int areaW) {

        this.areaW = areaW;
    }

    /**
     * @return the areaH
     */
    public int getAreaH() {

        return this.areaH;
    }

    /**
     * @param areaH
     *            the areaH to set
     */
    public void setAreaH(final int areaH) {

        this.areaH = areaH;
    }

    /**
     * Return the region as <code>Rectangle</code>
     *
     * @return a <code>Rectangle</code> for this region
     */
    public Rectangle getRectangle() {

        final int recX = (int) (this.areaX * AbstractBankPDFPage.CM_TO_PDF);
        final int recY = (int) (this.areaY * AbstractBankPDFPage.CM_TO_PDF);
        final int recW = (int) (this.areaW * AbstractBankPDFPage.CM_TO_PDF);
        final int recH = (int) (this.areaH * AbstractBankPDFPage.CM_TO_PDF);
        return new Rectangle(recX, recY, recW, recH);
    }

    /**
     * Add this region to the given <code>PDFTextStripper</code>
     *
     * @param pdfTextStripper
     *            the PDFTextStripper
     * @param regionName
     *            the name of the region.
     */
    public void addToPDFTextStripperByArea(final PDFTextStripperByArea pdfTextStripper, final String regionName) {

        pdfTextStripper.addRegion(regionName, this.getRectangle());
    }

}
