/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.santander.assistant;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateInfo;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankEnum;
import de.frankmuenster.hibiscus.mahoe.utils.DebitCardValidator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DirectoryInput;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.hbci.HBCIProperties;
import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
import de.willuhn.jameica.hbci.gui.input.KontoInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
import de.willuhn.logging.Logger;

/**
 * Kontoanlage Dialog der Santander Bank
 *
 * @author Frank Mnster
 */
public class SantanderCreateAccountDialog extends AbstractCreateAccountDialog<SantanderCreateInfo> {

    /**
     * Benutername Input
     */
    private transient TextInput login;

    /**
     * Kontonummer Input
     */
    private transient TextInput ktoNummer;

    /**
     * Auszge speichern Input
     */
    private transient CheckboxInput savePDF;

    /**
     * Speicherordner Input
     */
    private transient DirectoryInput saveFolder;

    /**
     * Kontotyp
     */
    private transient SelectInput ktoTypeInput;

    /**
     * Ausgleichbuchung anlegen
     */
    private transient CheckboxInput createClearing;

    /**
     * Konto der Ausgleichbbuchung
     */
    private transient KontoInput clearingAccount;

    /**
     *
     */
    public SantanderCreateAccountDialog() {
	super();
	this.setTitle(I18N.tr("SantanderCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected SantanderCreateInfo getData() {

	final CreateInfo info = super.getData();

	this.data.setOwner(info.getOwner());
	this.data.setComment(info.getComment());
	this.data.setDescription(StringUtils.isEmpty(info.getDescription()) ? "Santander Bank" //$NON-NLS-1$
		: info.getDescription());

	return this.data;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

	super.paint(parent);

	final List<String> ktoTypen = new ArrayList<>();
	ktoTypen.add(I18N.tr("SantanderCreateAccountDialog.instantAccessSavingsAccount")); //$NON-NLS-1$
	ktoTypen.add(I18N.tr("SantanderCreateAccountDialog.creditcard")); //$NON-NLS-1$
	this.ktoTypeInput = new SelectInput(ktoTypen, ktoTypen.get(0));
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.kinfOfAccount"), this.ktoTypeInput); //$NON-NLS-1$

	this.login = new TextInput(StringUtils.EMPTY, 20, StringUtils.EMPTY);
	this.login.setComment(StringUtils.EMPTY);
	this.login.setMandatory(true);
	this.ktoNummer = new TextInput(StringUtils.EMPTY, 20, StringUtils.EMPTY);
	this.ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.differentAsLogin")); //$NON-NLS-1$
	this.accDescription.setHint("Santander Bank"); //$NON-NLS-1$

	this.savePDF = new CheckboxInput(false);
	this.saveFolder = new DirectoryInput(StringUtils.EMPTY);

	this.createClearing = new CheckboxInput(false);
	final KontoFilter ktoFilter = KontoFilter.createForeign(SynchronizeJobSepaUeberweisung.class);
	try {
	    this.clearingAccount = new KontoInput(null, ktoFilter);
	    this.clearingAccount.setComment(StringUtils.EMPTY);
	} catch (final RemoteException e) {
	    Logger.error(I18N.tr("SantanderCreateAccountDialog.createClearingAccountInputFailed")); //$NON-NLS-1$
	}

	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.login"), this.login); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.ktoNumber"), this.ktoNummer); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.lblsavePdf"), this.savePDF); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.lblsavePdfFolder"), this.saveFolder); //$NON-NLS-1$

	// Ausgleichsbuchung
	this.container.addHeadline(I18N.tr("SantanderCreateAccountDialog.onlyForCreditCardHeadLine")); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.createClearingLbl"), this.createClearing); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.clearingAccountLbl"), this.clearingAccount); //$NON-NLS-1$

	this.container.addText(" ", true); //$NON-NLS-1$
	final StringBuilder formString = new StringBuilder(300) // NOPMD
		.append("<form><p>") //$NON-NLS-1$ // NOPMD
		.append(I18N.tr("SantanderCreateAccountDialog.websiteHint")) //$NON-NLS-1$
		.append("<br/><a href=\"http://progs.frankmuenster.de/mahoe/santander\">http://progs.frankmuenster.de/mahoe/santander</a><br/>") //$NON-NLS-1$
		.append("</p></form>"); //$NON-NLS-1$

	final FormTextPart text = new FormTextPart();
	text.setText(formString.toString());
	this.container.addPart(text);

	this.setSize(610, 530);

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog#prepareData()
     */
    @Override
    protected boolean prepareData() {

	boolean dataOk = true;

	this.data = new SantanderCreateInfo();

	// Werte speichern
	this.data.setLogin((String) this.login.getValue());
	this.data.setKtoNummer((String) this.ktoNummer.getValue());
	if (StringUtils.isEmpty(this.data.getKtoNummer())) {
	    this.data.setKtoNummer(this.data.getLogin());
	}
	this.data.setSavePDF((Boolean) this.savePDF.getValue());
	this.data.setSaveFolder((String) this.saveFolder.getValue());
	this.data.setBankType(SantanderBankEnum.CONSUMER);
	this.data.setCreateClearing((Boolean) this.createClearing.getValue());
	this.data.setClearingAccount((Konto) this.clearingAccount.getValue());

	if ("Tagesgeldkonto".equals(this.ktoTypeInput.getValue())) { //$NON-NLS-1$
	    this.data.setAccountType(KontoType.GIRO);
	    // Plausibilittsprfung
	    if (StringUtils.isEmpty(this.data.getKtoNummer())
		    || !HBCIProperties.checkAccountCRC(SantanderBank.BLZ, this.data.getKtoNummer())) {
		dataOk = false;
		this.ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.accountNumberWrong")); //$NON-NLS-1$
	    } else {
		this.ktoNummer.setComment(StringUtils.EMPTY);
	    }
	} else {
	    this.data.setAccountType(KontoType.KREDITKARTE);
	    // Plausibilittsprfung
	    if (StringUtils.isEmpty(this.data.getKtoNummer())
		    || !DebitCardValidator.isValidateCardNumber(this.data.getKtoNummer())) {
		dataOk = false;
		this.ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.creditcardNumberWrong")); //$NON-NLS-1$
	    } else {
		this.ktoNummer.setComment(StringUtils.EMPTY);
	    }
	    if (this.data.isCreateClearing() && this.data.getClearingAccount() == null) {
		this.clearingAccount.setComment(I18N.tr("SantanderCreateAccountDialog.fieldRequired")); //$NON-NLS-1$
		dataOk = false;
	    } else {
		this.clearingAccount.setComment(StringUtils.EMPTY);
	    }
	}

	return dataOk;

    }

}
