
package de.frankmuenster.hibiscus.mahoe.santander.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobProviderKontoauszug;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.utils.BankToAdd;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;

/**
 * Implementierung des Job-Providers fr den Abruf von Kontoauszuegen der BOS.
 *
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
@BankToAdd(blz = SantanderBank.BLZ, name = SantanderBank.NAME)
public class SantanderSynchronizeJobProviderKontoauszug extends AbstractMahoeSynchronizeJobProviderKontoauszug {

    /**
     * Standardkonstruktor
     */
    public SantanderSynchronizeJobProviderKontoauszug() {
        super();
    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#getJobs()
     */
    @Override
    protected List<Class<? extends SynchronizeJob>> getJobs() {

        final List<Class<? extends SynchronizeJob>> list = new ArrayList<>(1);
        list.add(SantanderSynchronizeJobKontoauszug.class);
        return list;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#getSupportedAccountTypes()
     */
    @Override
    protected Set<KontoType> getSupportedAccountTypes() {

        final Set<KontoType> set = new HashSet<>(1);
        // add supported account types
        set.add(KontoType.GIRO);
        set.add(KontoType.KREDITKARTE);
        return set;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#isBankSupported(Konto)
     */
    @Override
    protected boolean isBankSupported(final Konto account) throws RemoteException {

        return SantanderBank.BLZ.equals(account.getBLZ());
    }

}
