
package de.frankmuenster.hibiscus.mahoe.santander.web;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.logging.Logger;

/**
 * Enum fr die Santander Banken Consumer und Direkt
 *
 * @author Frank Mnster
 */
public enum SantanderBankEnum {

    /**
     * Consumer Bank
     */
    CONSUMER(
            "consumer", //$NON-NLS-1$
            "Consumer Bank",  //$NON-NLS-1$
            // "https://securebank.santander.de/IRALOG_SCB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_logonPinUser",
            // //$NON-NLS-1$
            "https://securebank.santander.de/IRALOG_UNILOGON_ENS/BtoChannelDriver.ssobto?dse_operationName=UniLogon",  //$NON-NLS-1$
            "https://securebank.santander.de/EBANDE_PGSCB_ENS/BtoChannelDriver.ssobto?dse_operationName=globalPositionOwnerOP", //$NON-NLS-1$
            "https://securebank.santander.de/EBANDE_LGSSCB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_inBox"), //$NON-NLS-1$

    /**
     * Direkt Bank
     */
    DIREKT(
            "direkt", //$NON-NLS-1$
            "Direkt Bank",  //$NON-NLS-1$
            "https://onlinebanking.santander-direkt.de/IRALOG_SDB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_logonPinUser", //$NON-NLS-1$
            "https://onlinebanking.santander-direkt.de/EBANDE_PGSDB_ENS/BtoChannelDriver.ssobto?dse_operationName=globalPositionOwnerOP", //$NON-NLS-1$
            "https://onlinebanking.santander-direkt.de/EBANDE_LGSSDB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_inBox"); //$NON-NLS-1$

    /**
     * Der Identifikations-String
     */
    private String idString;

    /**
     * Der Name der Bank
     */
    private String bankName;

    /**
     * Login URL
     */
    private String loginUrl;

    /**
     * &Uuml;bersichtseite URL
     */
    private String overviewUrl;

    /**
     * PostBox URL
     */
    private String postBoxUrl;

    /**
     * Aufz&auml;hlung der Santanderbanken
     *
     * @param idString
     *            der Identifikations-String.
     * @param bankName
     *            der Name der Bank.
     * @param loginUrl
     *            die Login URL
     * @param overviewUrl
     *            die Finanz&uuml;bersicht URL
     * @param postBoxUrl
     *            die PostBox URL
     */
    private SantanderBankEnum(final String idString, final String bankName, final String loginUrl,
            final String overviewUrl, final String postBoxUrl) {
        this.idString = idString;
        this.bankName = bankName;
        this.loginUrl = loginUrl;
        this.overviewUrl = overviewUrl;
        this.postBoxUrl = postBoxUrl;
    }

    /**
     * Liefert den Identifikations-String.
     *
     * @return idString der Identifikations-String.
     */
    public String getIdString() {

        return this.idString;
    }

    /**
     * Liefert den Bank Namen.
     *
     * @return den Bank Namen.
     */
    public String getBankName() {

        return this.bankName;
    }

    /**
     * Liefert die Login URL.
     *
     * @return die Login URL.
     */
    public String getLoginUrl() {

        return this.loginUrl;
    }

    /**
     * Liefert die Login URL.
     *
     * @return die Login URL.
     */
    public String getOverviewUrl() {

        return this.overviewUrl;
    }

    /**
     * Liefert die PostBox URL.
     *
     * @return die PostBox URL.
     */
    public String getPostBoxUrl() {

        return this.postBoxUrl;
    }

    /**
     * Ermittelt BankTyp.
     *
     * @param idString
     *            der Bank. Kann NULL sein.
     * @return der ID-String oder NULL, wenn der Code nicht bekannt ist.
     */
    public static SantanderBankEnum find(final String idString) {

        SantanderBankEnum santanderBank = null;
        if (StringUtils.trimToNull(idString) != null) {
            for (final SantanderBankEnum bankName : SantanderBankEnum.values()) {
                if (bankName.getIdString().equals(idString)) {
                    santanderBank = bankName;
                    break;
                }
            }
        }
        return santanderBank;
    }

    /**
     * Liefert das ENUM des Banktyps (CONSUMER, DIREKT).
     *
     * @param konto
     *            das zu untersuchende Konto
     * @return den BankTypen oder NULL
     */
    public static SantanderBankEnum find(final Konto konto) {

        String bankType = null;
        try {
            bankType = konto.getMeta(MahoeSynchronizeBackend.BANK_NAME, StringUtils.EMPTY);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractSantanderWebSite.readBankNameFailed"), e); //$NON-NLS-1$
        }
        return SantanderBankEnum.find(bankType);
    }

    /**
     * @see java.lang.Enum#toString()
     */
    @Override
    public String toString() {

        return this.getBankName();
    }
}
