/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.santander.web;

import java.io.IOException;
import java.nio.charset.Charset;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.WebResponse;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSelect;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung fr ein Tagesgeldkonto.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports" })
public final class SantanderWebSiteGiro
        extends AbstractSantanderWebSite<SantanderWebAccountInformation, SantanderBankMessages> {

    /**
     * CSV Felderanzahl
     */
    private static final int CSV_FIELDS = 33;

    /**
     *
     */
    private static final String TAGESGELDKONTEN = "Tagesgeldkonten"; //$NON-NLS-1$

    /**
     * L&auml;d eine Zeile aus der CSV-Datei in einen Umsatz.
     *
     * @param transaction
     *            die Umsatzzeile aus dem CSV
     * @return den Umsatz
     * @throws RemoteException
     */
    private Umsatz loadLine(final String[] transaction) // NOPMD
            throws RemoteException {

        // Buchungsdatum-Wertstellung-Transaktionsart-Betrag (EUR)-Saldo (EUR)-
        // Verwendungszweck 1-Verwendungszweck 2-Verwendungszweck
        // 3-Verwendungszweck 4-
        // Verwendungszweck 5-Verwendungszweck 6-Verwendungszweck
        // 7-Verwendungszweck 8-
        // Verwendungszweck 9-Verwendungszweck 10-Verwendungszweck
        // 11-Verwendungszweck 12-
        // Verwendungszweck 13-Verwendungszweck 14-Verwendungszweck 15-
        final HBCIDBService dbService = Settings.getDBService();
        final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
                Locale.getDefault());
        final Umsatz umsatz = dbService.createObject(Umsatz.class, null);
        // Buchungsdatum
        Date date = null;
        try {
            date = sdf.parse(transaction[0]);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.parseBookingDateFailed", e.getMessage()), e); //$NON-NLS-1$
        }
        umsatz.setDatum(date);
        // Valuta
        try {
            date = sdf.parse(transaction[1]);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.parseValutaDateFailed", e.getMessage()), e); //$NON-NLS-1$
        }
        umsatz.setValuta(date);
        // Transaktionsart
        umsatz.setArt(transaction[2]);
        // Betrag in EUR
        umsatz.setBetrag(Utils.convertStringToDouble(transaction[3]));
        // Saldo
        if (StringUtils.isNotEmpty(transaction[4])) {
            umsatz.setSaldo(Utils.convertStringToDouble(transaction[4]));
        }
        // Zweck
        final StringBuilder zweck = new StringBuilder();
        for (int i = 5; i < 19; i++) {
            zweck.append(transaction[i]).append('\n');
        }
        zweck.append(transaction[19]);
        Utils.loadUmsatz(umsatz, zweck.toString());
        return umsatz;
    }

    /**
     * Konstruktor fr ein Tagesgeldkonto.
     *
     * @param konto
     * @param userName
     * @throws WebSiteException
     */
    public SantanderWebSiteGiro(final Konto konto, final String userName) throws WebSiteException {

        super(konto, userName);
    }

    /**
     * Liefert die weiteren Kontodetails
     *
     * @return die Kontodetails
     * @throws WebSiteException
     */
    @Override
    public SantanderWebAccountInformation getAccountInformation() throws WebSiteException {

        try {
            final String ktoNummer = this.accountInfo.getKonto().getKontonummer();
            // get anchor to details page
            final HtmlAnchor details = super.getAccountDetailsAnchor(ktoNummer, SantanderWebSiteGiro.TAGESGELDKONTEN,
                    AbstractSantanderWebSite.ANCHOR_CELL_DETAILS);

            if (details == null) {
                throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.detailsPageNotFound"), //$NON-NLS-1$
                        WebSiteExceptionTypes.PAGE_NOT_FOUND);
            } else {
                final HtmlPage detailsPage = details.click();
                // IBAN
                final HtmlElement iban = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[5]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setIban(iban.asText());
                // BIC
                final HtmlElement bic = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[6]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setBic(bic.asText());
                // Saldo
                final HtmlElement saldo = (HtmlElement) detailsPage
                        .getByXPath("/html/body/div/div[3]/div[1]/div/div[1]/div/div[7]/span[2]").get(0); //$NON-NLS-1$
                this.accountInfo.setSaldo(Utils.convertStringToDouble(saldo.asText()));
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.readAccountOverviewFailedLog", //$NON-NLS-1$
                    e.getMessage()), e);
            throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.readAccountOverviewFailed"), // NOPMD //$NON-NLS-1$
                    WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
        }
        return this.accountInfo;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        try {
            // get account number
            final String ktoNummer = this.accountInfo.getKonto().getKontonummer();
            // get transaction anchor
            final HtmlAnchor anchor = super.getAccountDetailsAnchor(ktoNummer, SantanderWebSiteGiro.TAGESGELDKONTEN,
                    AbstractSantanderWebSite.ANCHOR_CELL_TRANSACTIONS);
            if (anchor != null) {
                // anchor to account transactions found, now load
                // and give back
                return this.getTransactions(anchor);
            }
            throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.transactionsPageNotFound"), //$NON-NLS-1$
                    WebSiteExceptionTypes.PAGE_NOT_FOUND);
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("SantanderWebSiteGiro.readBookFailedLog", e.getMessage())); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("SantanderWebSiteGiro.readBookFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.HTTP_STATUS_CODE, e);
        }

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        // Gibt es bei Tagesgeld nicht
        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#savePDF(de.frankmuenster.hibiscus.mahoe.santander.web.SantanderWebAccountInformation)
     */
    @Override
    public int savePDF(final SantanderWebAccountInformation accountInfo) throws PdfSaverException {

        Logger.info(I18N.tr("AbstractSantanderWebSite.saveSheetsNotSupported")); //$NON-NLS-1$
        return 0;
    }

    /**
     * Reads all transactions from the given anchor page
     *
     * @param anchor
     * @return eine Liste mit den gefundenen Ums&auml;tzen
     * @throws IOException
     */
    private List<Umsatz> getTransactions(final HtmlAnchor anchor) throws IOException {

        final List<Umsatz> transactions = new ArrayList<>();

        // open account
        HtmlPage transPage = anchor.click();

        final HtmlAnchor anchorSearch = transPage.getAnchorByText("(Einklappen / Ausklappen)"); //$NON-NLS-1$

        if (anchorSearch != null) {

            // open search form
            transPage = anchorSearch.click();

            final HtmlForm formSearch = transPage.getFormByName("accInfoReg"); //$NON-NLS-1$

            // select last 180 days
            final HtmlSelect timeperiodSelect = formSearch.getSelectByName("accInfoReg.days"); //$NON-NLS-1$
            timeperiodSelect.setSelectedAttribute("-180", true); //$NON-NLS-1$

            // find and click search submit button
            final HtmlInput submitShow = formSearch.getInputByName("accInfoReg.linkButton.events.0"); //$NON-NLS-1$
            final HtmlPage pageList = submitShow.click();

            // Now search export anchor
            final List<HtmlAnchor> anchors = pageList.getAnchors();
            HtmlAnchor anchorExport = null;
            for (int j = 0; j < anchors.size(); j++) {
                anchorExport = anchors.get(j);
                if ("export".equalsIgnoreCase(anchorExport.getTextContent())) { //$NON-NLS-1$
                    // read transactions from CSV file
                    this.readTransactionsFromCSV(anchorExport, transactions);
                }
            }
        }
        return transactions;
    }

    /**
     * Reads all transactions from the provided link to the CSV export.
     *
     * @param anchorExport
     *            link to the CSV export
     * @param transactions
     *            the list of add the transactions to
     * @throws IOException
     *             if an IO error occurs
     */
    private void readTransactionsFromCSV(final HtmlAnchor anchorExport, final List<Umsatz> transactions)
            throws IOException {

        final Page csvExport = anchorExport.click();
        final WebResponse response = csvExport.getWebResponse();
        Logger.debug("ContentType: " + response.getContentType()); //$NON-NLS-1$
        final String responseAsString = response.getContentAsString(Charset.forName("ISO-8859-1")); //$NON-NLS-1$
        // Split in lines
        final String[] lines = responseAsString.split("\n"); //$NON-NLS-1$
        for (final String line : lines) {
            final String[] transaction = line.split(";", -1); //$NON-NLS-1$
            if (transaction.length == SantanderWebSiteGiro.CSV_FIELDS) {
                if (transaction[0].equals("Buchungsdatum")) { //$NON-NLS-1$ //
                    continue;
                }
                final Umsatz umsatz = this.loadLine(transaction);
                transactions.add(umsatz);
            }
        }
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setzte das startDatum auf jetzt minus 180 Tage
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.add(Calendar.DAY_OF_YEAR, -180);
        return startCalendar;
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite#isSupporting(de.willuhn.jameica.hbci.rmi.KontoType)
     */
    @Override
    protected boolean isSupporting(final KontoType type) {

        return KontoType.GIRO.equals(type);
    }
}
