/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.targobank.assistant;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateInfo;
import de.frankmuenster.hibiscus.mahoe.utils.DebitCardValidator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DirectoryInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.hbci.rmi.KontoType;

/**
 * Dialog zum Abfragen der MahoeMain Kreditkarteninformationen
 *
 * @author Frank Mnster
 */
public class TargoBankCreateAccountDialog extends AbstractCreateAccountDialog<TargoBankCreateInfo> {

    /**
     * Kartennummer Input
     */
    private transient TextInput cardNumber;

    /**
     * Kartenkonto Input
     */
    private transient TextInput cardAccount;

    /**
     * Anbieter Input
     */
    private transient TextInput cardCompany;

    /**
     * Login Input
     */
    private transient TextInput login;

    /**
     * Auszge speichern Input
     */
    private transient CheckboxInput savePDF;

    /**
     * Speicherordner Input
     */
    private transient DirectoryInput saveFolder;

    /**
     */
    public TargoBankCreateAccountDialog() {

	super();
	this.setTitle(I18N.tr("TargoBankCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected TargoBankCreateInfo getData() {

	final CreateInfo info = super.getData();

	this.data.setOwner(info.getOwner());
	this.data.setComment(info.getComment());
	this.data.setAccountType(KontoType.KREDITKARTE);

	return this.data;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

	super.paint(parent);

	this.cardNumber = new TextInput(StringUtils.EMPTY, 20, "5500 0000 0000 0004"); //$NON-NLS-1$
	this.cardNumber.setMandatory(true);
	this.cardNumber.setComment(I18N.tr("TargoBankCreateAccountDialog.cardNumberComment")); //$NON-NLS-1$

	this.cardAccount = new TextInput(StringUtils.EMPTY, 20, "5500 0000 0000 0004"); //$NON-NLS-1$
	this.cardAccount.setComment(I18N.tr("TargoBankCreateAccountDialog.cardAccountComment")); //$NON-NLS-1$

	this.cardCompany = new TextInput(StringUtils.EMPTY, 50, "Select MasterCard"); //$NON-NLS-1$
	this.cardCompany.setMandatory(true);
	this.cardCompany.setComment(I18N.tr("TargoBankCreateAccountDialog.companyComment")); //$NON-NLS-1$

	this.login = new TextInput(StringUtils.EMPTY, 50, "Kundenkennung"); //$NON-NLS-1$
	this.login.setMandatory(true);
	this.login.setComment(I18N.tr("TargoBankCreateAccountDialog.loginComment")); //$NON-NLS-1$

	this.savePDF = new CheckboxInput(true);
	this.saveFolder = new DirectoryInput(StringUtils.EMPTY);

	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardNumber"), this.cardNumber); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardAccount"), this.cardAccount); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblCardCompany"), this.cardCompany); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblLogin"), this.login); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblsavePdf"), this.savePDF); //$NON-NLS-1$
	this.container.addLabelPair(I18N.tr("TargoBankCreateAccountDialog.lblsavePdfFolder"), this.saveFolder); //$NON-NLS-1$

	this.container.addText(" ", true); //$NON-NLS-1$
	final StringBuilder formString = new StringBuilder(300).append("<form><p>") // NOPMD //$NON-NLS-1$
		.append(I18N.tr("TargoBankCreateAccountDialog.websiteHint")) //$NON-NLS-1$
		.append("<br/><a href=\"http://progs.frankmuenster.de/mahoe/targo-bank\">http://progs.frankmuenster.de/mahoe/targo-bank</a><br/></p></form>"); //$NON-NLS-1$
	final FormTextPart text = new FormTextPart();
	text.setText(formString.toString());
	this.container.addPart(text);

	this.setSize(610, 460);

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateAccountDialog#prepareData()
     */
    @Override
    protected boolean prepareData() {

	boolean dataOK = true;

	this.data = new TargoBankCreateInfo();
	this.data.setCardNumber((String) this.cardNumber.getValue());
	this.data.setCardAccount((String) this.cardAccount.getValue());
	if (StringUtils.isEmpty(this.data.getCardAccount())) {
	    this.data.setCardAccount(this.data.getCardNumber());
	}
	this.data.setCompany((String) this.cardCompany.getValue());
	this.data.setLogin((String) this.login.getValue());
	this.data.setSavePDF((Boolean) this.savePDF.getValue());
	this.data.setSaveFolder((String) this.saveFolder.getValue());

	if (StringUtils.isEmpty(this.data.getCardNumber())
		|| !DebitCardValidator.isValidateCardNumber(this.data.getCardNumber())) {
	    dataOK = false;
	    this.cardNumber.setComment(I18N.tr("TargoBankCreateAccountDialog.cardNumberWrong")); //$NON-NLS-1$
	} else {
	    this.cardNumber.setComment(StringUtils.EMPTY);
	}
	if (StringUtils.isEmpty(this.data.getCompany())) {
	    dataOK = false;
	    this.cardCompany.setComment(I18N.tr("TargoBankCreateAccountDialog.cardTypeWrong")); //$NON-NLS-1$
	} else {
	    this.cardCompany.setComment(StringUtils.EMPTY);
	}
	if (StringUtils.isEmpty(this.data.getLogin())) {
	    dataOK = false;
	    this.login.setComment(I18N.tr("TargoBankCreateAccountDialog.loginWrong")); //$NON-NLS-1$
	} else {
	    this.login.setComment(StringUtils.EMPTY);
	}

	return dataOK;

    }
}
