/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.targobank.jobs; // NOPMD 

import java.util.Arrays;
import java.util.List;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.targobank.web.TargoBankMessages;
import de.frankmuenster.hibiscus.mahoe.targobank.web.TargoBankWebAccountInformation;
import de.frankmuenster.hibiscus.mahoe.targobank.web.TargoBankWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.utils.TransactionSaver;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hier wird die eigentliche Arbeit gemacht.
 *
 * @author Frank Mnster
 */
public class TargoBankSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<TargoBankWebAccountInformation, TargoBankMessages> {

    /**
     * Standartkonstruktor
     */
    public TargoBankSynchronizeJobKontoauszug() {

        super();
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = TargoBankWebSite.getInstance(this.konto);
    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        final char[] password = PasswdCache.getPassword(TargoBank.NAME, this.kundennummer);

        try {
            this.webSite.login(password);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
                // Passwort zurcksetzten
                PasswdCache.revoke(TargoBank.NAME, this.kundennummer);
            }
            throw e;
        }
        Arrays.fill(password, ' ');

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        // Lade vorgemerkte Umstze
        final List<Umsatz> preBooked = this.webSite.getPreBookedTransactions();
        this.monitor.setPercentComplete(30);

        this.monitor.setStatusText(
                I18N.tr("SynchronizeJobKontoauszug.getBooked", this.accountInfo.getLastTransactionDateAsString())); //$NON-NLS-1$
        final List<Umsatz> booked = this.webSite.getBookedTransactions();
        this.monitor.setPercentComplete(40);

        // Debug-Ausgabe
        if (Logger.isLogging(Level.DEBUG)) {
            this.debugLogTransactions(preBooked);
            this.debugLogTransactions(booked);
        }

        // Speicher vorgemerkte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingPreBooked")); //$NON-NLS-1$
        final int preBookedCount = TransactionSaver.savePreBooked(this.konto, preBooked);
        this.monitor
                .setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(45);

        // Speicher gebuchte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
        final int bookedCount = TransactionSaver.saveBooked(this.konto, booked);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(50);

        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

    }

    // /**
    // * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#handleAccountSheets()
    // */
    // @Override
    // protected void handleAccountSheets() throws WebSiteException, ApplicationException {
    //
    // try {
    // final boolean savePdf = Utils
    // .getBooleanFromYesNoString(this.konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF, Utils.TEXT_NEIN));
    // if (savePdf) {
    // Logger.info(I18N.tr("TargoBankSynchronizeJobKontoauszug.savePDF")); //$NON-NLS-1$
    // String folder = this.konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, StringUtils.EMPTY);
    // if (StringUtils.isEmpty(folder)) {
    // // get new folder by opening a folder dialog box
    // folder = getPdfSaveFolderLocationFromUser();
    // }
    // if (StringUtils.isEmpty(folder)) {
    // Logger.error(I18N.tr("TargoBankSynchronizeJobKontoauszug.noSavePathGiven")); //$NON-NLS-1$
    // } else {
    // this.accountInfo.setSavePath(folder);
    // final int savedCount = this.webSite.savePDF(this.accountInfo);
    // if (savedCount == 0) {
    // this.monitor.setStatusText(I18N.tr("TargoBankSynchronizeJobKontoauszug.noNewAccountSheets")); //$NON-NLS-1$
    // } else {
    // this.monitor
    // .setStatusText(I18N
    // .tr("TargoBankSynchronizeJobKontoauszug.savingNewSheetsIn", Integer.toString(savedCount), folder)); //$NON-NLS-1$
    // final String text = I18N.tr("TargoBankSynchronizeJobKontoauszug.newAccountSheetsAvailableMsg"); //$NON-NLS-1$
    // Application.getMessagingFactory().sendMessage(
    // new StatusBarMessage(text, StatusBarMessage.TYPE_SUCCESS)); // NOPMD
    // }
    // this.monitor.setPercentComplete(90);
    // }
    // } else {
    // this.monitor.setStatusText(I18N.tr("TargoBankSynchronizeJobKontoauszug.savePdfDisabled")); //$NON-NLS-1$
    // Logger.info(I18N.tr("TargoBankSynchronizeJobKontoauszug.savePdfDisabled")); //$NON-NLS-1$
    // this.monitor.setPercentComplete(90);
    // }
    // } catch (RemoteException | PdfSaverException e) {
    // Logger.error(I18N.tr("TargoBankSynchronizeJobKontoauszug.savePdfFailed"), e); //$NON-NLS-1$
    // throw new ApplicationException(e.getMessage(), e);
    // }
    // }
}
