
package de.frankmuenster.hibiscus.mahoe.targobank.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobProviderKontoauszug;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;

/**
 * Implementierung des Job-Providers fr den Abruf von Kontoauszgen.
 *
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
public class TargoBankSynchronizeJobProviderKontoauszug extends AbstractMahoeSynchronizeJobProviderKontoauszug {

    /**
     * Standardkonstruktor
     */
    public TargoBankSynchronizeJobProviderKontoauszug() {
        super();
    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#getJobs()
     */
    @Override
    protected List<Class<? extends SynchronizeJob>> getJobs() {

        final List<Class<? extends SynchronizeJob>> jobs = new ArrayList<>();
        jobs.add(TargoBankSynchronizeJobKontoauszug.class);
        return jobs;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#getSupportedAccountTypes()
     */
    @Override
    protected Set<KontoType> getSupportedAccountTypes() {

        final Set<KontoType> set = new HashSet<>(1);
        // add supported account types
        set.add(KontoType.KREDITKARTE);

        return set;

    }

    /**
     * @see AbstractMahoeSynchronizeJobProviderKontoauszug#isBankSupported(Konto)
     */
    @Override
    protected boolean isBankSupported(final Konto account) throws RemoteException {

        return TargoBank.BLZ.equals(account.getBLZ());
    }

}
