
package de.frankmuenster.hibiscus.mahoe.targobank.web;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Bean mit Kontoinformationen von der Webseite
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class TargoBankWebAccountInformation extends AbstractWebAccountInformation {

    /**
     * Das Kartenkonto
     */
    private String cardAccount;

    /**
     * Der Kartenanbieter
     */
    private String cardCompany;

    /**
     * Die Kartennummer
     */
    private String cardNumber;

    /**
     * Nchster Flligkeitstermin
     */
    private Date nextPayDay;

    /**
     * Der Verfgungsrahmen
     */
    private Double verfuegungsrahmen;

    /**
     * Standardkonstruktor
     */
    public TargoBankWebAccountInformation() {
        super();
    }

    /**
     * @return the cardAccount
     */
    public String getCardAccount() {

        return this.cardAccount;
    }

    /**
     * @return the cardCompany
     */
    public String getCardCompany() {

        return this.cardCompany;
    }

    /**
     * @return the cardNumber
     */
    public String getCardNumber() {

        return this.cardNumber;
    }

    /**
     * @return the nextPayDay
     */
    public Date getNextPayDay() {

        return this.nextPayDay;
    }

    /**
     * @return the verfuegungsrahmen
     */
    public Double getVerfuegungsrahmen() {

        return this.verfuegungsrahmen;
    }

    /**
     * @param cardAccount
     *            the cardAccount to set
     */
    public void setCardAccount(final String cardAccount) {

        this.cardAccount = cardAccount;
    }

    /**
     * @param cardCompany
     *            the cardCompany to set
     */
    public void setCardCompany(final String cardCompany) {

        this.cardCompany = cardCompany;
    }

    /**
     * @param cardNumber
     *            the cardNumber to set
     */
    public void setCardNumber(final String cardNumber) {

        this.cardNumber = cardNumber;
    }

    /**
     * @param nextPayDay
     *            the nextPayDay to set
     */
    public void setNextPayDay(final Date nextPayDay) {

        this.nextPayDay = nextPayDay;
        this.updateComment();

    }

    /**
     * @param verfuegungsrahmen
     *            the verfuegungsrahmen to set
     */
    public void setVerfuegungsrahmen(final Double verfuegungsrahmen) {

        this.verfuegungsrahmen = verfuegungsrahmen;
        this.updateComment();
    }

    private void updateComment() {

        final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
                Locale.GERMAN);

        final String rahmen = this.verfuegungsrahmen == null ? StringUtils.EMPTY
                : Double.toString(this.verfuegungsrahmen);
        final String payDay = this.nextPayDay == null ? StringUtils.EMPTY : sdf.format(this.nextPayDay);

        final String kommentar = I18N.tr("TargoBankSynchronizeJobKontoauszug.comment", //$NON-NLS-1$
                rahmen, payDay);

        this.setComment(kommentar);

    }

    /**
     * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebAccountInformation#saveBankSpecificInfo(de.willuhn.util.ProgressMonitor)
     */
    @Override
    protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {

        // Noch nix zu tun
    }
}
