/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.web;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Calendar;
import java.util.Date;

import com.gargoylesoftware.htmlunit.WebClient;

import de.frankmuenster.hibiscus.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Abstrakte WebSite Klasse.
 *
 * @author Frank Mnster
 * @param <I>
 * @param <M>
 */
public abstract class AbstractWebSite<I extends WebAccountInformation, M extends AbstractBankMessages>
        implements WebSite<I> {

    /**
     * HTML Tag einer Tabelle
     */
    protected static final String TABLE_TAG = "table"; //$NON-NLS-1$

    /**
     * Non breaking space unicode
     */
    public static final String NON_BREAK_SPACE = "\u00a0"; //$NON-NLS-1$

    /**
     * Unser WebClient
     */
    protected WebClient webClient;

    /**
     * Die Accountinfos
     */
    protected I accountInfo;

    /**
     * Unser Konstruktor. Nur zugnglich fr Sub-Klassen.
     *
     * @param konto
     *            das Konto
     * @throws WebSiteException
     */
    protected AbstractWebSite(final Konto konto) throws WebSiteException {

        java.util.logging.Logger.getLogger("com.gargoylesoftware.htmlunit") //$NON-NLS-1$
                .setLevel(java.util.logging.Level.OFF);
        this.webClient = this.getWebClient();

        this.setAccountBasicInfo(konto);

    }

    /**
     * Ldt neue Bank-Nachrichten.
     *
     * @param accountInfo
     * @throws WebSiteException
     * @throws ApplicationException
     */
    @Override
    public int getMessages(final I accountInfo) throws WebSiteException, ApplicationException {

        int saveMsg;

        final ParameterizedType superClass = (ParameterizedType) this.getClass().getGenericSuperclass();
        final Type type = superClass.getActualTypeArguments()[1];
        @SuppressWarnings("unchecked")
        final Class<M> clazz = (Class<M>) type;

        Logger.debug(clazz.toString());

        try {
            final M messages = clazz.getDeclaredConstructor(WebClient.class).newInstance(this.webClient);

            messages.readMessages(accountInfo.getLastTransactionDate());
            saveMsg = messages.saveMessage();

        } catch (final ApplicationException e) {
            Logger.error(I18N.tr("AbstractWebSite.getBankMessagesFailed"), //$NON-NLS-1$
                    e);
            throw e;
        } catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException
                | NoSuchMethodException | SecurityException e) {
            Logger.error(I18N.tr("AbstractWebSite.instanziateBankMessageClassFailed"), //$NON-NLS-1$
                    e);
            throw new ApplicationException(I18N.tr("AbstractWebSite.instanziateBankMessageClassFailed"), //$NON-NLS-1$
                    e);
        }

        return saveMsg;
    }

    /**
     * Das Datum, ab dem Umstze und Banknachrichten abgerufen werden sollen, falls noch keine Umstze vorliegen.
     *
     * @return das StartDatum
     */
    protected abstract Calendar getStartdate();

    /**
     * Erstellt den bankspezifischen WebClient.
     *
     * @return den HtmlUnit WebClient
     */
    protected abstract WebClient getWebClient();

    private void setAccountBasicInfo(final Konto konto) throws WebSiteException {

        final ParameterizedType superClass = (ParameterizedType) this.getClass().getGenericSuperclass();
        final Type type = superClass.getActualTypeArguments()[0];
        @SuppressWarnings("unchecked")
        final Class<I> clazz = (Class<I>) type;

        try {
            this.accountInfo = clazz.newInstance();

            // Setze Konto
            this.accountInfo.setKonto(konto);

            // Setzte letztes Umsatzdatum
            final Date lastTransacDate = Utils.getLastUmsatzDate(konto, this.getStartdate());
            this.accountInfo.setLastTransactionDate(lastTransacDate);

        } catch (InstantiationException | IllegalAccessException e) {
            Logger.error(I18N.tr("AbstractWebSite.settingAccountInformationFailed")); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("AbstractWebSite.settingAccountInformationFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
        }
    }

}
