/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.hibiscus.mahoe.web;

import java.util.Date;

import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Interface fr die Kontoinformationen, die von der Webseite gelesen/gesetzt werden.
 *
 * @author Frank Mnster
 */
public interface WebAccountInformation {

    /**
     * Sets the saldo available.
     *
     * @param saldoAvailable
     *            the new saldo available
     */
    void setSaldoAvailable(final Double saldoAvailable);

    /**
     * Sets the saldo.
     *
     * @param saldo
     *            the new saldo
     */
    void setSaldo(final Double saldo);

    /**
     * Sets the last transaction date.
     *
     * @param lastTransactionDate
     *            the new last transaction date
     */
    void setLastTransactionDate(final Date lastTransactionDate);

    /**
     * Sets the konto.
     *
     * @param konto
     *            the new konto
     */
    void setKonto(final Konto konto);

    /**
     * Sets the comment.
     *
     * @param comment
     *            the new comment
     */
    void setComment(final String comment);

    /**
     * Gets the saldo available.
     *
     * @return saldoAvailable
     */
    Double getSaldoAvailable();

    /**
     * Gets the saldo.
     *
     * @return saldo
     */
    Double getSaldo();

    /**
     * Gets the last transaction date as string.
     *
     * @return lastTransactionDate als String
     */
    String getLastTransactionDateAsString();

    /**
     * Gets the last transaction date.
     *
     * @return lastTransactionDate
     */
    Date getLastTransactionDate();

    /**
     * Gets the konto.
     *
     * @return konto
     */
    Konto getKonto();

    /**
     * Gets the comment.
     *
     * @return comment
     */
    String getComment();

    /**
     * Speichert die Web-Kontoinformationen in das Konto.
     *
     * @param monitor
     *            der ProgressMonitor
     * @throws ApplicationException
     *             the application exception
     */
    void saveInformationInAccount(ProgressMonitor monitor) throws ApplicationException;

    /**
     * Sets the save path.
     *
     * @param savePath
     *            the new save path
     */
    void setSavePath(final String savePath);

    /**
     * Gets the save path.
     *
     * @return the save path
     */
    String getSavePath();

}
