
package de.frankmuenster.hibiscus.mahoe.web;

/**
 * Spezielle Fehlerklasse fr Webseitenfehler
 *
 * @author Frank Mnster
 */
public class WebSiteException extends Exception {

    /**
     * Die Serial UID
     */
    private static final long serialVersionUID = 3682535546017597734L;

    /**
     * Der Type des Yahoo Fehlers
     */
    private final transient WebSiteExceptionTypes type;

    /**
     * Konstruktor mit Informationstext und Fehlertyp.
     * 
     * @param message
     *            der Informationstext
     * @param type
     *            der Fehlertyp
     */
    public WebSiteException(final String message, final WebSiteExceptionTypes type) {
        this(message, type, null);
    }

    /**
     * Konstruktor mit Informationstext und Fehlertyp.
     * 
     * @param message
     *            der Informationstext
     * @param cause
     *            der Grund
     */
    public WebSiteException(final String message, final Throwable cause) {
        this(message, null, cause);
    }

    /**
     * Konstruktor mit Informationstext und Fehlertyp.
     * 
     * @param message
     *            der Informationstext
     * @param type
     *            der Fehlertyp
     * @param cause
     *            der Grund
     */
    public WebSiteException(final String message, final WebSiteExceptionTypes type, final Throwable cause) {
        super(message, cause);
        this.type = type;
    }

    /**
     * Liefert den Fehlertyp der Exception
     * 
     * @return den Webseiten-Fehlertyp
     */
    public WebSiteExceptionTypes getType() {

        return this.type;
    }
}
